#include <string.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>

void saddlept_id( double *, double *, int *, double *, double *,
                  int *, double *, double *, double *, int *,
                  double (*)( double, double * ), double (*)( double,
                  double * ), double (*)( double, double * ) );

void calc_sdi_id( double *lb, double *sp_lb, double *sp_ub,
                  double *epsrel, double *epsabs, double *max_abserr,
                  double *theta, int *max_iter, double *sdi, int *err,
                  double (*k)( double, double * ), double (*dk)( double,
                  double * ), double (*ddk)( double, double * ) )
{
   int err_qagiu=0;
   double err_sdi=*max_abserr+1.0;
   struct fparams { double sp_lb, sp_ub, epsrel, epsabs, sp, *theta,
                    (*k)( double, double * ), (*dk)( double, double * ),
                    (*ddk)( double, double * ); int lt, max_iter, sig; }
                    sp_eval;
   double f( double x, void *p )
   {
      struct fparams *fp = (struct fparams *) p;

      saddlept_id( &fp->sp_lb, &fp->sp_ub, &fp->lt, &x, fp->theta,
                   &fp->max_iter, &fp->epsrel, &fp->epsabs, &fp->sp,
                   &fp->sig, fp->k, fp->dk, fp->ddk );
      if( !gsl_finite( fp->sp ) || fp->sig ) {
         return( 0.0 );
      }
      return( fp->sp );
   }

   gsl_error_handler_t *gsl_errh;
   gsl_integration_workspace *w;
   gsl_function F;

   sp_eval.sp_lb = *sp_lb; sp_eval.sp_ub = *sp_ub;
   sp_eval.epsrel = *epsrel; sp_eval.epsabs = *epsabs;
   sp_eval.theta = theta;
   sp_eval.k = k; sp_eval.dk = dk; sp_eval.ddk = ddk;
   sp_eval.lt = 1; sp_eval.max_iter = *max_iter;

   w = gsl_integration_workspace_alloc( 1000 );
   F.function = &f;
   F.params = (void *) &sp_eval;
   gsl_errh = gsl_set_error_handler_off();
   err_qagiu = gsl_integration_qagiu( &F, *lb, *epsabs, *epsrel, 1000, w,
                                      sdi, &err_sdi );
   gsl_set_error_handler( gsl_errh );
   gsl_integration_workspace_free( w );

   if( err_sdi > *max_abserr ) {
      *err = 1; *sdi = 0; return;
   }
   *err = 0;
   return;
}
