#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_math.h>

double calc_M( int *, double *, double *, int *, double (*)( double, double * ),
               double (*)( double, double * ) );
double calc_m( int *, double *, double *, double (*)( double, double * ) );

void calc_spb_id( double *m, double *M, double *theta, int *N, int *ntheta,
                  double *sp_lb, double *sp_ub, int *err,
                  double (*k)( double, double * ),
                  double (*dk)( double, double * ) )
{
   int i, j, max_ub=1000, max_lb=1000;
   double *th;

   th = (double *) malloc( *ntheta * sizeof( double ) );

   for( i=0; i<*N; i++ ) {
      for( j=0; j<*ntheta; j++ ) {
         th[j] = theta[j+i*(*ntheta)];
      }

   /* Find upper bound for root-finder. */

      sp_ub[i] = calc_M( &max_ub, &M[i], th, err, k, dk );
      if( *err ) {
         free( th ); return;
      }

   /* Find lower bound for root-finder. */

      sp_lb[i] = calc_m( &max_lb, &m[i], th, dk );
   }
   free( th );
   return;
}

double calc_M( int *max_ub, double *M, double *theta, int *err_M, double (*k)
               ( double, double * ), double (*dk) ( double, double * ) )
{
   int iter, iiter, err, max_iub=10;
   double s, sc, ub, eps, ks, ksc, dks, dksc;

   s = 0.0; sc = 0.0; ub = s; eps = 1.0e-3;
   ksc = 0.0; dksc = dk( s, theta );
   *err_M = 1 - gsl_finite( dksc );
   if( *err_M ) {
      return( s );
   }
   s += eps; iter = 0;
   while( iter < *max_ub ) {
      iiter = 0;
      while( iiter < max_iub ) {
         ks = k( s, theta ); dks = dk( s, theta );
         err = gsl_finite( ks ) + gsl_finite( dks );
         if( err < 2 ) {
            iiter++; s -= eps; eps /= 100.0; s += eps;
         }
         else {
            break;
         }
      }
      if( err < 2 ) {
         iter++;
      }
      else {
         if( dksc >= dks || ksc >= ks ) {
            iter++; s -= eps; eps /= 100.0; s += eps;
         }
         else if( dks >= *M ) {
            return( s );
         }
         else {
            sc = s; ksc = ks; dksc = dks;
            s += eps;
         }
      }
   }
   *M = dksc;
   return( sc );
}

double calc_m( int *max_lb, double *m, double *theta, double (*dk)
               ( double, double * ) )
{
   int iter, iiter, err, max_ilb=10;
   double eps, logs, logsc, dks, dksc;

   eps = 2.0; logs = 0.0;
   dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
   while( !err ) {
      logs -= eps;
      dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
   }
   if( dks < *m ) {
      return( -exp( logs ) );
   }
   logsc = logs; dksc = dks;

   iter = 0; eps = 1.0; logs += eps;
   while( iter < *max_lb ) {
      iiter = 0;
      while( iiter < max_ilb ) {
         dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
         if( !err ) {
            iiter++; logs -= eps; eps /= 2.0; logs += eps;
         }
         else {
            break;
         }
      }
      if( !err ) {
         iter++;
      }
      else {
         if( dks >= dksc ) {
            iter++; logs -= eps; eps /= 2.0; logs += eps;
         }
         else if( dks < *m ) {
            return( -exp( logs ) );
         }
         else {
            logsc = logs; dksc = dks;
            logs += eps;
         }
      }
   }
   *m = dksc + 1e-8;
   return( -exp( logsc ) );
}
