#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_sum.h>

void fourier_id( int *M, double *A, int *lt, double *t, double *theta,
                 int *N, int *ntheta, int *max_iter, double *eps,
                 double *fi, int *err_fi, double (*m)( gsl_complex,
                 double * ))
{
   int i, j, k, iter, Nl, Nu;
   double pi, *th, *fid, *a, sum_accel, err;
   gsl_complex z;
   gsl_sum_levin_utrunc_workspace *w;

   pi = 4.0 * atan( 1.0 );

   th = (double *) malloc( *ntheta * sizeof( double ) );
   fid = (double *) malloc( *lt * sizeof( double ) );

   for( i=0; i<*N; i++ ) {
      for( j=0; j<*ntheta; j++ ) {
         th[j] = theta[j+i*(*ntheta)];
      }
      for( j=0; j<*lt; j++ ) {
         err = GSL_POSINF;
         GSL_SET_COMPLEX( &z, -(*A)/(2.0*t[j]), 0.0 );
         a = malloc( 1 * sizeof( double ) );
         a[0] = m( z, th )/2.0;
         iter = 0; Nl = 1; Nu = *M;
         while( err > *eps && iter < *max_iter ) {
            a = realloc( a, (Nu+1)*sizeof( double ) );
            w = gsl_sum_levin_utrunc_alloc( Nu+1 );
            for (k=Nl; k<=Nu; k++) {
               GSL_SET_COMPLEX( &z, -(*A)/(2.0*t[j]), -(k*pi)/t[j] );
               a[k] = pow( -1, k )*m( z, th );
            }
            gsl_sum_levin_utrunc_accel( a, Nu+1, w, &sum_accel, &err );
            gsl_sum_levin_utrunc_free( w );
            iter += 1; Nl += *M; Nu += *M;
         }
         free( a );
         fid[j] = (exp( (*A)/2.0 )/t[j]) * sum_accel;
         fid[j] = (fid[j] >= 0.0) ? fid[j] : 0.0;
         fi[j] += fid[j];
         if (iter == *max_iter) { *err_fi = 1; }
      }
   }
   free( fid ); free( th );
   return;
}
