#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_math.h>

void calc_sdi_id( double *, double *, double *, double *, double *,
                  double *, double *, int *, double *, int *,
                  double (*)( double, double * ), double (*)( double,
                  double * ), double (*)( double, double * ) );
void saddlept_id( double *, double *, int *, double *, double *,
                  int *, double *, double *, double *, int *,
                  double (*)( double, double * ), double (*)( double,
                  double * ), double (*)(double, double * ) );

void saddle_id( double *sp_lb, double *sp_ub, int *lt, double *t,
                double *theta, int *N, int *ntheta, int *max_iter, 
                double *epsrel, double *epsabs, double *max_abserr,
                double *sp, double *nc, int *err_sd, double (*k)( double,
                double * ), double (*dk)( double, double * ),
                double (*ddk)( double, double * ) )
{
   int i, j, err_nc, *sig;
   double lb=0.0, *th, *w;

   th = (double *) malloc( *ntheta * sizeof( double ) );
   w = (double *) malloc( *lt * sizeof( double ) );
   sig = (int *) calloc( *lt, sizeof( int ) );

   for( i=0; i<*N; i++ ) {
      for( j=0; j<*ntheta; j++ ) {
         th[j] = theta[j+i*(*ntheta)];
      }

/* Saddlepoint approximations */

      saddlept_id( &sp_lb[i], &sp_ub[i], lt, t, th, max_iter, epsrel, epsabs,
                   w, sig, k, dk, ddk );
      for( j=0; j<*lt; j++ ) {
         if( !gsl_finite( w[j] ) || sig[j] ) {
            w[j] = 0.0;
         }
      }

/* Normalizing constants */

      calc_sdi_id( &lb, &sp_lb[i], &sp_ub[i], epsrel, epsabs, max_abserr,
                   th, max_iter, &nc[i], &err_nc, k, dk, ddk );
      if( nc[i] <= 0.0 || !gsl_finite( nc[i] ) || err_nc ) {
          *err_sd = 1; free( sig ); free( w ); free( th );
          return;
      }

      for( j=0; j<*lt; j++ ) {
         sp[j] += (w[j] / nc[i]);
      }
   }

   free( sig ); free( w ); free( th );
   return;
}
