#include <math.h>
#include <gsl/gsl_roots.h>

void root( double *, double *, double *, double *, int *, gsl_root_fsolver *,
           gsl_function *, double *, int * );

void saddlept_id( double *sp_lb, double *sp_ub, int *lt, double *t,
                  double *theta, int *max_iter, double *epsrel,
                  double *epsabs, double *sp, int *sig,
                  double (*k)( double, double * ),
                  double (*dk)( double, double * ),
                  double (*ddk)( double, double * ) )
{
   int i, err_rt;
   double lb=*sp_lb, pi;
   struct fparams { double t, *theta, (*dk)( double, double * ); } fp;
   double f( double s, void *p )
   {
      struct fparams *fp = (struct fparams *)p;
      return fp->dk( s, fp->theta ) - fp->t;
   }
   const gsl_root_fsolver_type *T;
   gsl_root_fsolver *rf;
   gsl_function F;

   pi = 4.0 * atan( 1.0 );

   fp.theta = theta; fp.dk = dk;

   F.function = &f;
   F.params = &fp;

   T = gsl_root_fsolver_brent;
   rf = gsl_root_fsolver_alloc( T );

   for( i=0; i<*lt; i++ ) {

      fp.t = t[i];

   /* Run root-finder. */

      root( &lb, sp_ub, epsrel, epsabs, max_iter, rf, &F, &sp[i], &err_rt );

      if( !err_rt ) {
         lb = sp[i];
         sp[i] = -0.5 * (log( 2*pi ) + log(ddk( sp[i], theta ))) +
                 k( sp[i], theta ) - sp[i] * t[i];
         sp[i] = exp( sp[i] );
         sig[i] = 0;
      }
      else if( err_rt == 1 ) {
         sp[i] = -1.0; sig[i] = 1;
      }
      else {
         sp[i] = -1.0; sig[i] = 2;
      }
   }
   gsl_root_fsolver_free( rf );
   return;
}
