#include <string.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>

void fourier( int *, double *, int *, double *, double *, int *, int *,
              int *, int *, double *, int *, double *, int *, int * );

void calc_fdi_lu( double *lb, double *ub, double *epsrel, double *epsabs,
                  double *max_abserr, int *M, double *A, int *ntheta,
                  double *theta, int *max_iter, double *eps, double *fdi,
                  int *err )
{
   size_t neval;
   int err_qng=0, err_qag=0;
   double err_fdi=*max_abserr+1.0;
   struct fparams { double A, eps, *theta, fd;
                    int M, lt, N, ntheta, max_iter, info, file, err_fi; }
                    fi_eval;
   double f( double x, void *p )
   {
      int nthread=1;
      struct fparams *fi = (struct fparams *) p;
      fi->fd = 0.0; fi->err_fi = 0;
      fourier( &fi->M, &fi->A, &fi->lt, &x, fi->theta, &fi->N,
               &fi->ntheta, &fi->max_iter, &nthread, &fi->eps, &fi->info,
               &fi->fd, &fi->file, &fi->err_fi );
      if( !gsl_finite( fi->fd ) || fi->err_fi ) {
         return( 0.0 );
      }
      return( fi->fd );
   }

   gsl_error_handler_t *gsl_errh;
   gsl_integration_workspace *w;
   gsl_function F;

   fi_eval.M = *M; fi_eval.A = *A;
   fi_eval.lt = 1; fi_eval.file = 0;
   fi_eval.theta = theta;
   fi_eval.N = 1; fi_eval.ntheta = *ntheta;
   fi_eval.max_iter = *max_iter; fi_eval.eps = *eps;
   fi_eval.info = 0;

   F.function = &f;
   F.params = (void *) &fi_eval;
   gsl_errh = gsl_set_error_handler_off();
   err_qng = gsl_integration_qng( &F, *lb, *ub, *epsabs, *epsrel, fdi,
                                  &err_fdi, &neval );
   if( err_fdi > *max_abserr ) {
      w = gsl_integration_workspace_alloc( 1000 );
      err_qag = gsl_integration_qag( &F, *lb, *ub, *epsabs, *epsrel, 1000,
                                     6, w, fdi, &err_fdi );
      gsl_integration_workspace_free( w );
   }
   gsl_set_error_handler( gsl_errh );

   if( err_fdi > *max_abserr ) {
      *err = 1; *fdi = 0; return;
   }
   *err = 0;
   return;
}
