#include <string.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>

void saddlept( double *, double *, int *, double *, double *,
               int *, double *, double *, double *, int * );

void calc_sdi_lu( double *lb, double *ub, double *sp_lb, double *sp_ub,
                  double *epsrel, double *epsabs, double *max_abserr,
                  double *theta, int *max_iter, double *sdi, int *err )
{
   size_t neval;
   int err_qng=0, err_qag=0;
   double err_sdi=*max_abserr+1.0;
   struct fparams { double sp_lb, sp_ub, epsrel, epsabs, sp, *theta;
                    int lt, max_iter, sig; } sp_eval;
   double f( double x, void *p )
   {
      struct fparams *fp = (struct fparams *) p;
      fp->sig = 0;
      saddlept( &fp->sp_lb, &fp->sp_ub, &fp->lt, &x, fp->theta, &fp->max_iter,
                &fp->epsrel, &fp->epsabs, &fp->sp, &fp->sig );
      if( !gsl_finite( fp->sp ) || fp->sig ) {
         return( 0.0 );
      }
      return( fp->sp );
   }

   gsl_error_handler_t *gsl_errh;
   gsl_integration_workspace *w;
   gsl_function F;

   sp_eval.sp_lb = *sp_lb; sp_eval.sp_ub = *sp_ub;
   sp_eval.epsrel = *epsrel; sp_eval.epsabs = *epsabs;
   sp_eval.theta = theta;
   sp_eval.lt = 1; sp_eval.max_iter = *max_iter;

   F.function = &f;
   F.params = (void *) &sp_eval;
   gsl_errh = gsl_set_error_handler_off();
   err_qng = gsl_integration_qng( &F, *lb, *ub, *epsabs, *epsrel, sdi,
                                  &err_sdi, &neval );
   if( err_sdi > *max_abserr ) {
      w = gsl_integration_workspace_alloc( 1000 );
      err_qag = gsl_integration_qag( &F, *lb, *ub, *epsabs, *epsrel, 1000,
                                     6, w, sdi, &err_sdi );
      gsl_integration_workspace_free( w );
   }
   gsl_set_error_handler( gsl_errh );

   if( err_sdi > *max_abserr ) {
      *err = 1; *sdi = 0; return;
   }
   *err = 0;
   return;
}
