#include <stdlib.h>
#include <math.h>
#include <pthread.h>
#include <gsl/gsl_math.h>

double k( double, double * );
double dk( double, double * );
double ddk( double, double * );

double calc_M( int *, double *, double * );
double calc_m( int *, double *, double * );

void calc_spb( double *m, double *M, double *theta, int *N,
               int *ntheta, int *nthread, double *sp_lb, double *sp_ub )
{
   int i, mm, MM, iret[*nthread];
   pthread_t threads[*nthread];

   struct pthr { int ntheta, mm, MM; double *theta, *m, *M, *sp_lb,
                 *sp_ub; }
               thr[*nthread];
   void *speval( void *p )
   {
      int i, j, max_ub=1000, max_lb=1000;
      double *th;
      struct pthr *pt = (struct pthr *) p;

      th = (double *) malloc( pt->ntheta * sizeof( double ) );
      for( i=pt->mm; i<=pt->MM; i++ ) {
         for( j=0; j<pt->ntheta; j++ ) {
            th[j] = pt->theta[j+i*(pt->ntheta)];
         }
         /* Find upper bound for root-finder. */
         pt->sp_ub[i] = calc_M( &max_ub, &pt->M[i], th );
         /* Find lower bound for root-finder. */
         pt->sp_lb[i] = calc_m( &max_lb, &pt->m[i], th );
      }
      free( th );
      pthread_exit( NULL );
   }

   for( i=0; i<*nthread; i++ ) {
      thr[i].ntheta = *ntheta; thr[i].theta = theta;
      thr[i].m = m; thr[i].M = M;
      thr[i].sp_lb = sp_lb; thr[i].sp_ub = sp_ub;
      if( *nthread == 1 ) { mm = 0; MM = *N-1; }
      else {
         if( i == 0 ) { mm = 0; MM = *N / *nthread - 1; }
         else if( i == *nthread-1 ) { mm = MM+1; MM = *N-1; }
         else { mm = MM+1; MM = ((i+1) * (*N)) / *nthread - 1; }
      }
      thr[i].mm = mm; thr[i].MM = MM;
      iret[i] = pthread_create( &threads[i], NULL, speval, (void *) &thr[i] );
   }
   for( i=0; i<*nthread; i++ ) {
      pthread_join( threads[i], NULL );
   }
   return;
}

double calc_M( int *max_ub, double *M, double *theta )
{
   int iter, iiter, err, max_iub=10;
   double s, sc, ub, eps, ks, ksc, dks, dksc;

   s = 0.0; sc = 0.0; ub = s; eps = 1.0e-3;
   ksc = 0.0; dksc = dk( s, theta );
   s += eps; iter = 0;
   while( iter < *max_ub ) {
      iiter = 0;
      while( iiter < max_iub ) {
         ks = k( s, theta ); dks = dk( s, theta );
         err = gsl_finite( ks ) + gsl_finite( dks );
         if( err < 2 ) {
            iiter++; s -= eps; eps /= 100.0; s += eps;
         }
         else {
            break;
         }
      }
      if( err < 2 ) {
         iter++;
      }
      else {
         if( dksc >= dks || ksc >= ks ) {
            iter++; s -= eps; eps /= 100.0; s += eps;
         }
         else if( dks >= *M ) {
            return( s );
         }
         else {
            sc = s; ksc = ks; dksc = dks;
            s += eps;
         }
      }
   }
   *M = dksc;
   return( sc );
}

double calc_m( int *max_lb, double *m, double *theta )
{
   int iter, iiter, err, max_ilb=10;
   double eps, logs, logsc, dks, dksc;

   eps = 2.0; logs = 0.0;
   dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
   while( !err ) {
      logs -= eps;
      dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
   }
   if( dks < *m ) {
      return( -exp( logs ) );
   }
   logsc = logs; dksc = dks;

   iter = 0; eps = 1.0; logs += eps;
   while( iter < *max_lb ) {
      iiter = 0;
      while( iiter < max_ilb ) {
         dks = dk( -exp( logs ), theta ); err = gsl_finite( dks );
         if( !err ) {
            iiter++; logs -= eps; eps /= 2.0; logs += eps;
         }
         else {
            break;
         }
      }
      if( !err ) {
         iter++;
      }
      else {
         if( dks >= dksc ) {
            iter++; logs -= eps; eps /= 2.0; logs += eps;
         }
         else if( dks < *m ) {
            return( -exp( logs ) );
         }
         else {
            logsc = logs; dksc = dks;
            logs += eps;
         }
      }
   }
   *m = dksc + 1e-8;
   return( -exp( logsc ) );
}
