#
# Read in Inverse Gaussian functions.
#

source("../inv_gauss.r")

#
# Set simulation parameters.
#

n <- 62          # Number of patients.

p02 <- 0.17      # Transitions from 0
p01 <- 1 - p02

l1 <- 7; l2 <- 0.5   # 0 -> 1 waiting time IG parameters
r1 <- 0.013          # 0 -> 2 waiting time E parameters
r2 <- 0.067          # 1 -> 2 waiting time E parameters

censor <- numeric(n)
visits <- numeric(n)
tottime <- numeric(n)

#
# Simulate censoring times
#

#ctime <- rep(-1, n)

sct <- 1
mct <- 46
ctime <- rnorm(n, mean=mct, sd=sct)

data <- NULL
for (i in 1:n) {
   states <- numeric(); states[1] <- 0;
   time <- numeric(); time[1] <- 0;
   cs <- 0; j <- 1;
   while (1) {
      if (cs == 0) {
         x <- rbinom(1, size=1, prob=p01)
         if (x) {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rinvgauss(1, a=l1, b=l2); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r1); j <- j+1;
            break;
         }
      }
      else if (cs == 1) {
         cs <- 2; states[j+1] <- cs;
         time[j+1] <- rexp(1, rate=r2); j <- j+1;
         break;
      }
   }
   out <- numeric(); time <- cumsum(time);
   for (k in 1:(j-1)) {
      out[2*k-1] <- states[k]; out[2*k] <- time[k+1];
      if (ctime[i] >= time[k] && ctime[i] < time[k+1]) {
         out[2*k+1] <- states[k]; out[2*k] <- ctime[i];
         censor[i] <- 1; j <- k+1; break;
      }
   }
   if (!censor[i]) { out[2*j-1] <- states[j]; }
   states <- NULL; time <- NULL; data <- c(data, out);
   tottime[i] <- out[2*j-2]; out <- NULL; visits[i] <- j;
}

data.df <- data.frame(visits=visits, tottime=tottime, censor=censor)
write.table(data.df, file="data_summary", quote=FALSE)
write(data, file="data_complete", ncol=1)
