#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include "../log_density.h"

void transform( double *, int *, double * );

double post( double *theta, int *ip, double *dp )
{

   int i, err, pp=ip[6]+ip[1]+3;
   double pdf, sf1, sf2, tmp, *ttheta, *p=theta;

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      for( i=0; i<ip[1]; i++ ) {
         ttheta[i] = theta[i];
      }
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {
/* Prior 2 Boundaries. */
      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[6]+i] ) {
            return( HUGE_VAL );
         }
      }
      if ( p[0] >= dp[ip[6]+ip[1]] || p[3] >= dp[ip[6]+ip[1]+1] ||
           p[4] >= dp[ip[6]+ip[1]+2] ) {
         return( HUGE_VAL );
      }
   }

/* Calculate likelihood */

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      pdf = log_invgauss_pdf( dp[i], p[3], p[4] );
      tmp += pdf + log(1-p[0]);
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[1] );
      tmp += pdf + log(p[0]);
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[2] );
      tmp += pdf;
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      invgauss_sf( dp[i], p[3], p[4], &sf1 );
      sf1 *= (1-p[0]);
      sf2 = log_exponential_sf( dp[i], p[1] );
      sf2 = p[0]*exp(sf2);
      tmp += log( sf1 + sf2 );
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      sf1 = log_exponential_sf( dp[i], p[2] );
      tmp += sf1;
   }

/* Calculate prior 2 */
   for( i=1; i<3; i++ ) {
      pdf = log_gamma_pdf( p[i], dp[pp+2*i-2], dp[pp+2*i-1] );
      tmp += pdf;
   }

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}
