#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_ss"
ntheta <- 5; N <- 400; nburnin <- 1000; al <- .05;
m <- 1e-6; M <- 1e6;
xlim.up <- 150
t <- seq(m, xlim.up, 2)

#
# Read the data
#

dat1.df <- read.table("data_summary", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("data_complete")

id <- rep(1:nobs, 2*dat1.df$visits-1)
t02 <- t01 <- t12 <- c0 <- c1 <- NULL

for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) { t01 <- c(t01, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 2 ) { t02 <- c(t02, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 2 ) { t12 <- c(t12, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 0 ) { c0 <- c(c0, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 1 ) { c1 <- c(c1, d[2*j]-it) }
   }
}
      
ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t02); ip[4] <- ip[3]+length(t12);
ip[5] <- ip[4]+length(c0); ip[6] <- ip[5]+length(c1);
dp <- c(t01, t02, t12, c0, c1)

lb <- rep(0, ntheta)
ub <- c(1, 100, 100)
prior.p <- c(lb, ub, rep(1/100, 4))

dp <- c(dp, prior.p)

#
# Find MAP
#

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=MAP$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum)
Theta.ss <- Theta.ss[[1]]

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

Theta <- Theta.ss

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd,
                  nthread=3, outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
