#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_ss"
ntheta <- 5; N <- 2000; nburnin <- 1000; al <- .05;
m <- 1e-6; M <- 1e6;
xlim.up <- 50
t <- seq(m, xlim.up, 0.5)

#
# Read the data
#

t34 <- read.table("state34.txt", header=TRUE); t34 <- t34[,1];
t35 <- read.table("state35.txt", header=TRUE); t35 <- t35[,1];

ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t34);
ip[3] <- ip[2]+length(t35)
dp <- c(t34, t35)

lb <- numeric(ntheta)
ub <- c(1, rep(20, ntheta-1))
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

#
# Find MAP
#

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=MAP$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum)
Theta.ss <- Theta.ss[[1]]

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

Theta <- Theta.ss

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd,
                  nthread=3, outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, sf=sf, hf=hf, ylab=c("density", "cdf", "survivor",
      "hazard"), ulim=xlim.up, plot="all")
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up, lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="cdf", ulim=xlim.up, lwd=2, plot="cdf")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up, lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up, lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
