#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../fourier.r"); source("../sumstats_fi.r");
source("../mcvar.r"); source("../plots.r");

#
# Set-up
#

outsum <- "summary_ss"; outpd <- "mc_pd"; outcdf <- "mc_cdf";
outhf <- "mc_hf"; outmc <- "mc_ss";
ntheta <- 2; N <- 15000; nburnin <- 1000; al <- .05;
xlim.up <- 8.000001
t <- seq(1e-6, xlim.up, .05)
bch <- 0; nb <- 30;

M <- 1000; max.iter <- 10; A <- 20; eps <- 1e-8;

#
# Read the data
#

t08 <- read.table("data_08")
t08 <- table(sort(t08[,1]))
d08 <- as.numeric(t08)
t08 <- as.numeric(names(t08)); t08 <- t08 / 60;

ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t08);
ip[3] <- ip[2]+length(d08); ip[4] <- M; ip[5] <- max.iter;

dp <- c(t08, d08)

lb <- numeric(ntheta)
ub <- rep(10, ntheta)
prior.p <- c(lb, ub)
ip[6] <- length(prior.p)

dp <- c(dp, prior.p)

id <- c(A, eps)
dp <- c(dp, id)

save.image()

#
# Find MAP
#

system("cp post_map.c post.c")

MAP <- optimize(ntheta, ip, dp, maxiter=10000, id=1, outfile=outsum)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=MAP$theta, m=20, w=1,
                  alpha=al, nburn=nburnin, id=1, outfile=outsum,
                  file=TRUE)
Theta.ss <- as.matrix(read.table("theta.out"))

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

system("rm post.c")

save.image()

#
# Set Theta for Fourier inversion approximation and plots
#

Theta <- Theta.ss

#
# Fourier inversion approximation
#

FI <- fourier(N,Theta,t,M,A,max.iter,nthread=3,eps,info=1,file=TRUE)
pd <- FI$pd

save.image()

#
# Monte Carlo summaries
#

mc.pd <- read.table("pd.out")
pd.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(mc.pd[,i])$var) })

z.cv <- qnorm(0.975)
pd.lb <- pd - z.cv * pd.mcsd
pd.lb <- ifelse(pd.lb < 0, 0, pd.lb)
pd.ub <- pd + z.cv * pd.mcsd

write.table(cbind(pd.lb, pd, pd.ub), outpd, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

out <- sumstats(N, Theta, t, numeric(length(t)), task=1,
                outfile=outsum)
mu <- out$vdk; var <- out$vddk + (mu - mean(mu))^2;

save.image()

out <- sumstats(N, Theta, t, pd, task=2, nthread=3,
                outfile=outsum, file=TRUE)
cdf <- read.table("cdf.out")

save.image()

mu.mcmu <- mean(mu); var.mcmu <- mean(var+(mu-mu.mcmu)^2);
cdf.mcmu <- apply(cdf, 2, mean)

mu.mcsd <- sqrt(mcvar(mu)$var)
var.mcsd <- sqrt(mcvar(var+(mu-mu.mcmu)^2)$var)
cdf.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(cdf[,i])$var) })

cdf.lb <- cdf.mcmu - z.cv * cdf.mcsd
cdf.lb <- ifelse(cdf.lb < 0, 0, cdf.lb)
cdf.ub <- cdf.mcmu + z.cv * cdf.mcsd
cdf.ub <- ifelse(cdf.ub > 1, 1, cdf.ub)

write.table(cbind(cdf.lb, cdf.mcmu, cdf.ub), outcdf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

stats <- sumstats(N, Theta, t, pd, cdf=cdf.mcmu, task=3,
                  nthread=3, outfile=outsum)
q <- stats$q
xlim.up <- max(xlim.up, q[length(q)])

save.image()

sf <- 1 - cdf.mcmu; hf <- pd / sf;
hf.est <- t(sapply(1:nb,
          function (i) {
             sb <- N/nb
             iss <- (1+(i-1)*sb):(i*sb)
             pd.t <- apply(mc.pd[iss,],2,mean)
             cdf.t <- apply(cdf[iss,],2,mean)
             pd.t / (1 - cdf.t)
          }))
hf.mcmu <- apply(hf.est, 2, mean)
hf.mcsd <- sapply(1:length(t),
                   function (i) {
                      if (bch) {
                         hf.mcvar <- mcvar(hf.est[,i], b=nb, method="batch")
                         c(hf.mcvar$acf, sqrt(hf.mcvar$var))
                      } else {
                         hf.mcvar <- mcvar(hf.est[,i])
                         c(hf.mcvar$ess, sqrt(hf.mcvar$var))
                      }
                   })
if (bch) { hf.acf <- hf.mcsd[1,] } else { hf.ess <- hf.mcsd[1,] }
hf.mcsd <- hf.mcsd[2,]

hf.lb <- hf.mcmu - z.cv * hf.mcsd
hf.lb <- ifelse(hf.lb < 0, 0, hf.lb)
hf.ub <- hf.mcmu + z.cv * hf.mcsd

write.table(cbind(hf.lb, hf.mcmu, hf, hf.ub), outhf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

q.est <- t(sapply(1:nb,
         function (i) {
            sb <- N/nb
            iss <- (1+(i-1)*sb):(i*sb)
            cdf.t <- apply(cdf[iss,],2,mean)
            if (i == 1) {
               sumstats(sb, Theta[iss,], t, numeric(length(t)),
                        cdf=cdf.t, task=3, nthread=3, vb=F, clean=F)$q
            } else if (i < nb) {
               sumstats(sb, Theta[iss,], t, numeric(length(t)),
                        cdf=cdf.t, task=3, nthread=3, vb=F, compile=F,
                        clean=F)$q
            } else {
               sumstats(sb, Theta[iss,], t, numeric(length(t)),
                        cdf=cdf.t, task=3, nthread=3, vb=F, compile=F)$q
            }
         }))
rm(mc.pd, cdf)

save.image()

q.mcmu <- apply(q.est, 2, mean)
q.mcsd <- sapply(1:length(q),
                  function (i) {
                     if (bch) {
                        q.mcvar <- mcvar(q.est[,i], b=nb, method="batch")
                        c(q.mcvar$acf, sqrt(q.mcvar$var))
                     } else {
                        q.mcvar <- mcvar(q.est[,i])
                        c(q.mcvar$ess, sqrt(q.mcvar$var))
                     }
                  })
if (bch) { q.acf <- q.mcsd[1,] } else { q.ess <- q.mcsd[1,] }
q.mcsd <- q.mcsd[2,]

q.lb <- q.mcmu - z.cv * q.mcsd
q.lb <- ifelse(q.lb < 0, 0, q.lb)
q.ub <- q.mcmu + z.cv * q.mcsd

save.image()

write("Confidence Interval for PD Mean:\n", file=outmc)
write(c(mu.mcmu - z.cv * mu.mcsd, mu.mcmu, mu.mcmu + z.cv * mu.mcsd),
      file=outmc, ncol=3, append=TRUE)
write("\nConfidence Interval for PD Standard Deviation:\n",
      file=outmc, append=TRUE)
write(c(sqrt(var.mcmu - z.cv * var.mcsd), sqrt(var.mcmu),
        sqrt(var.mcmu + z.cv * var.mcsd)), file=outmc,
      ncol=3, append=TRUE)
write("\nConfidence Intervals for PD Quantiles:\n", file=outmc,
      append=TRUE)
write(t(cbind(q.lb, q.mcmu, q, q.ub)), file=outmc, ncol=4, append=TRUE)

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
