#
# Read in Inverse Gaussian functions.
#

source("../inv_gauss.r")

#
# Set simulation parameters.
#

n <- 100         # Number of patients.

p <- 0.4         # Transition probability

r1 <- 0.6                # 0 -> 1 waiting time E parameter
r2 <- 0.3                # 0 -> 2 waiting time E parameter
eta1 <- 6; eta2 <- 2;    # 0 -> 3 waiting time IG parameters

mu1 <- 0.5               # 1 -> 2, 2 -> 3 waiting time E parameter
mu2 <- 0.3               # 2 -> 1, 3 -> 2 waiting time E parameter

a <- 4; b <- 0.25;       # 1 -> 4, 2 -> 4, 3 -> 4 waiting time G parameters

del1 <- 4; del2 <- 7;    # 4 -> 5 waiting time IG parameters

censor <- numeric(n)
visits <- numeric(n)
tottime <- numeric(n)

#
# Simulate censoring times
#

ctime <- rep(-1, n)

#sct <- 1
#mct <- 46
#ctime <- rnorm(n, mean=mct, sd=sct)

data <- NULL
for (i in 1:n) {
   states <- numeric(); states[1] <- 0;
   time <- numeric(); time[1] <- 0;
   cs <- 0; j <- 1;
   while (1) {
      if (cs == 0) {
         x <- sample(3, 1, prob=c(1-p, p/2, p/2))
         if (x == 1) {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r1); j <- j+1;
            next;
         }
         else if (x == 2) {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r2); j <- j+1;
            next;
         }
         else {
            cs <- 3; states[j+1] <- cs;
            time[j+1] <- rinvgauss(1, a=eta1, b=eta2); j <- j+1;
            next;
         }
      }
      else if (cs == 1) {
         x <- rbinom(1, size=1, prob=p)
         if (x) {
            cs <- 4; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a, rate=b); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=mu1); j <- j+1;
            next;
         }
      }
      else if (cs == 2) {
         x <- sample(c(1, 4, 3), 1, prob=c((1-p)/2, p, (1-p)/2))
         if (x == 1) {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=mu2); j <- j+1;
            next;
         }
         else if (x == 4) {
            cs <- 4; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a, rate=b); j <- j+1;
            next;
         }
         else {
            cs <- 3; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=mu1); j <- j+1;
            next;
         }
      }
      else if (cs == 3) {
         x <- rbinom(1, size=1, prob=p)
         if (x) {
            cs <- 4; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a, rate=b); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=mu2); j <- j+1;
            next;
         }
      }
      else if (cs == 4) {
         cs <- 5; states[j+1] <- cs;
         time[j+1] <- rinvgauss(1, a=del1, b=del2); j <- j+1;
         break;
      }
   }
   out <- numeric(); time <- cumsum(time);
   for (k in 1:(j-1)) {
      out[2*k-1] <- states[k]; out[2*k] <- time[k+1];
      if (ctime[i] >= time[k] && ctime[i] < time[k+1]) {
         out[2*k+1] <- states[k]; out[2*k] <- ctime[i];
         censor[i] <- 1; j <- k+1; break;
      }
   }
   if (!censor[i]) { out[2*j-1] <- states[j]; }
   states <- NULL; time <- NULL; data <- c(data, out);
   tottime[i] <- out[2*j-2]; out <- NULL; visits[i] <- j;
}

data.df <- data.frame(visits=visits, tottime=tottime, censor=censor)
write.table(data.df, file="data_summary", quote=FALSE)
write(data, file="data_complete", ncol=1)
