#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_psi.h>
#include "../log_density.h"

void transform( double *, int *, double * );

double post( double *theta, int *ip, double *dp )
{
   int i, err;
   double pdf, tmp, *ttheta, *p=theta;

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      for( i=0; i<ip[1]; i++ ) {
         ttheta[i] = theta[i];
      }
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {
/* Boundaries for Prior. */

      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[12]+i] || p[i] >= dp[ip[12]+ip[1]+i] ) {
            return( HUGE_VAL );
         }
      }
   }

/* Calculate likelihood */

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[1] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[2] );
      tmp += pdf + log( p[0] / 2.0 );
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      pdf = log_invgauss_pdf( dp[i], p[3], p[4] );
      tmp += pdf + log( p[0] / 2.0 );
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[5] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      pdf = log_gamma_pdf( dp[i], p[7], p[8] );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[6]; i<ip[7]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[6] );
      tmp += pdf + log( (1.0 - p[0]) / 2.0 );
   }
   for( i=ip[7]; i<ip[8]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[5] );
      tmp += pdf + log( (1.0 - p[0]) / 2.0 );
   }
   for( i=ip[8]; i<ip[9]; i++ ) {
      pdf = log_gamma_pdf( dp[i], p[7], p[8] );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[9]; i<ip[10]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[6] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[10]; i<ip[11]; i++ ) {
      pdf = log_gamma_pdf( dp[i], p[7], p[8] );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[11]; i<ip[12]; i++ ) {
      pdf = log_invgauss_pdf( dp[i], p[9], p[10] );
      tmp += pdf;
   }

/* Jeffrey's prior */

   tmp += -log( p[1] ) - log( p[2] ) - log( p[3] ) / 2 - log( p[4] ) / 2 -
           log( p[5] ) - log( p[6] ) +
           log( p[7] * gsl_sf_psi_n( 1, p[7] ) - 1 ) / 2 - log( p[8] ) -
           log( p[9] ) / 2 - log( p[10] ) / 2;

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}
