#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_psi.h>
#include "../log_density.h"

void transform( double *, int *, double * );
void id_05( double, double, int *, double *, double *, int *, int *,
            double *, double *, double *, double *, int * );

double post( double *theta, int *ip, double *dp )
{

   int i, err=0, dp_id=ip[14]+ip[17];
   double pdf, tmp, wk, *ttheta, *p=theta, *t, *sp, *th;

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      for( i=0; i<ip[1]; i++ ) {
         ttheta[i] = theta[i];
      }
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {
/* Boundaries for Prior. */
      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[14]+i] || p[i] >= dp[ip[14]+ip[1]+i] ) {
            return( HUGE_VAL );
         }
      }
   }

/* Calculate likelihood */

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[1] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[2] );
      tmp += pdf + log( p[0] / 2.0 );
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      wk = sqrt( p[3] / p[4] );
      pdf = log_invgauss_pdf( dp[i], p[3] * wk, wk );
      tmp += pdf + log( p[0] / 2.0 );
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[5] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      wk = p[7] / p[8];
      pdf = log_gamma_pdf( dp[i], p[7] * wk, wk );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[6]; i<ip[7]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[6] );
      tmp += pdf + log( (1.0 - p[0]) / 2.0 );
   }
   for( i=ip[7]; i<ip[8]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[5] );
      tmp += pdf + log( (1.0 - p[0]) / 2.0 );
   }
   for( i=ip[8]; i<ip[9]; i++ ) {
      wk = p[7] / p[8];
      pdf = log_gamma_pdf( dp[i], p[7] * wk, wk );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[9]; i<ip[10]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[6] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[10]; i<ip[11]; i++ ) {
      wk = p[7] / p[8];
      pdf = log_gamma_pdf( dp[i], p[7] * wk, wk );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[11]; i<ip[12]; i++ ) {
      wk = sqrt( p[9] / p[10] );
      pdf = log_invgauss_pdf( dp[i], p[9] * wk, wk );
      tmp += pdf;
   }

/* Incomplete Data */
   t = (double *) malloc( ip[15] * sizeof( double ) );
   for( i=ip[12]; i<ip[13]; i++ ) {
      t[i-ip[12]] = dp[i];
   }
   sp = (double *) calloc( ip[15], sizeof( double ) );

   th = (double *) malloc( ip[1] * sizeof( double ) );
   th[0] = p[0]; th[1] = p[1]; th[2] = p[2];
   th[4] = sqrt( p[3] / p[4] ); th[3] = p[3] * th[4];
   th[5] = p[5]; th[6] = p[6];
   th[8] = p[7] / p[8]; th[7] = p[7] * th[8];
   th[10] = sqrt( p[9] / p[10] ); th[9] = p[9] * th[10];

   id_05( dp[dp_id], dp[dp_id+1], &ip[15], t, th, &ip[1],
          &ip[16], &dp[dp_id+2], &dp[dp_id+3], &dp[dp_id+4],
          sp, &err );

   free( th ); free( t );
   if( err ) {
      free( sp );
      if( ip[0] ) {
         free( ttheta );
      }
      return( HUGE_VAL );
   }
   for( i=ip[13]; i<ip[14]; i++ ) {
      tmp += dp[i] * log( sp[i-ip[13]] );
   }
   free( sp );

/* Jeffrey's prior */

   wk = pow( p[7], 2 ) / p[8];
   tmp += -log( p[1] ) - log( p[2] ) - 3 * log( p[4] ) / 2 -
           log( p[5] ) - log( p[6] ) + log( wk * gsl_sf_psi_n( 1, wk ) - 1 ) /
           2 + log( p[7] ) - 2 * log( p[8] ) - 3 * log( p[10] ) / 2; 

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}
