#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_ss"
ntheta <- 11; N <- 2000; nburnin <- 1000; al <- .05;
m <- 1e-6; M <- 1e4;
xlim.up <- 75
t <- seq(m, xlim.up, 0.5)

max.iter <- 1000; epsrel <- 1e-7; epsabs <- 0; max.abserr <- 1e-4;

#
# Read the data
#

dat1.df <- read.table("data_summary", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("data_complete")

# Count transitions / collect waiting times

id <- rep(1:nobs, 2*dat1.df$visits-1)
t01 <- t02 <- t03 <- t05 <- t12 <- t14 <- t21 <- t23 <- t24 <- t32 <-
t34 <- t45 <- NULL

for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) { t01 <- c(t01, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 2 ) { t02 <- c(t02, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 3 ) { t03 <- c(t03, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 5 ) { t05 <- c(t05, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 2 ) { t12 <- c(t12, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 4 ) { t14 <- c(t14, d[2*j]-it) }
      if( d[2*j-1] == 2 && d[2*j+1] == 1 ) { t21 <- c(t21, d[2*j]-it) }
      if( d[2*j-1] == 2 && d[2*j+1] == 3 ) { t23 <- c(t23, d[2*j]-it) }
      if( d[2*j-1] == 2 && d[2*j+1] == 4 ) { t24 <- c(t24, d[2*j]-it) }
      if( d[2*j-1] == 3 && d[2*j+1] == 2 ) { t32 <- c(t32, d[2*j]-it) }
      if( d[2*j-1] == 3 && d[2*j+1] == 4 ) { t34 <- c(t34, d[2*j]-it) }
      if( d[2*j-1] == 4 && d[2*j+1] == 5 ) { t45 <- c(t45, d[2*j]-it) }
   }
}

t05 <- table(sort(t05))
d05 <- as.numeric(t05)
t05 <- as.numeric(names(t05))

ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t02); ip[4] <- ip[3]+length(t03);
ip[5] <- ip[4]+length(t12); ip[6] <- ip[5]+length(t14);
ip[7] <- ip[6]+length(t21); ip[8] <- ip[7]+length(t23);
ip[9] <- ip[8]+length(t24); ip[10] <- ip[9]+length(t32);
ip[11] <- ip[10]+length(t34); ip[12] <- ip[11]+length(t45);
ip[13] <- ip[12]+length(t05); ip[14] <- ip[13]+length(d05);
ip[15] <- length(t05); ip[16] <- max.iter;
dp <- c(t01, t02, t03, t12, t14, t21, t23, t24, t32, t34, t45,
        t05, d05)

lb <- numeric(ntheta)
ub <- c(1, rep(1000, ntheta-1))

prior.p <- c(lb, ub)
ip[17] <- length(prior.p)

dp <- c(dp, prior.p)

id <- c(m, M, epsrel, epsabs, max.abserr)
dp <- c(dp, id)

#
# Find MAP
#

system("cp post_map.c post.c")

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum, id=1)

system("rm post.c")

system("cp post_ss.c post.c")

SS <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum, id=1)

save.image()

#
# Slice sampling
#

Theta.ss <- slice(N, ntheta, ip, dp, start=SS$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum, id=1)
Theta.ss <- Theta.ss[[1]]

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

system("rm post.c")

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

if (nburnin > 0) {
   Theta.ss.burnin[,5] <- sqrt(Theta.ss.burnin[,4] / Theta.ss.burnin[,5])
   Theta.ss.burnin[,4] <- Theta.ss.burnin[,4] * Theta.ss.burnin[,5]
   Theta.ss.burnin[,9] <- Theta.ss.burnin[,8] / Theta.ss.burnin[,9]
   Theta.ss.burnin[,8] <- Theta.ss.burnin[,8] * Theta.ss.burnin[,9]
   Theta.ss.burnin[,11] <- sqrt(Theta.ss.burnin[,10] / Theta.ss.burnin[,11])
   Theta.ss.burnin[,10] <- Theta.ss.burnin[,10] * Theta.ss.burnin[,11]
}

Theta.ss[,5] <- sqrt(Theta.ss[,4] / Theta.ss[,5])
Theta.ss[,4] <- Theta.ss[,4] * Theta.ss[,5]
Theta.ss[,9] <- Theta.ss[,8] / Theta.ss[,9]
Theta.ss[,8] <- Theta.ss[,8] * Theta.ss[,9]
Theta.ss[,11] <- sqrt(Theta.ss[,10] / Theta.ss[,11])
Theta.ss[,10] <- Theta.ss[,10] * Theta.ss[,11]

Theta <- Theta.ss

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd,
                  nthread=3, outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
graphics.off()
