#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_sum.h>

double m( gsl_complex, double * );

void fourier( int *M, double *A, int *lt, double *t, double *theta,
              int *N, int *ntheta, int *max_iter, double *eps,
              int *info, double *fi, int *err_fi, int *file)
{
   int i, j, k, iter, Nl, Nu;
   double pi, *th, *fid, *a, sum_accel, err;
   gsl_complex z;
   gsl_sum_levin_utrunc_workspace *w;
   FILE *f;

   pi = 4.0 * atan( 1.0 );

   th = (double *) malloc( *ntheta * sizeof( double ) );
   fid = (double *) malloc( *lt * sizeof( double ) );
   if( *file ) {
      f = fopen( "psf.out", "a" );
   }

   for( i=0; i<*N; i++ ) {
      for( j=0; j<*ntheta; j++ ) {
         th[j] = theta[j+i*(*ntheta)];
      }
      for( j=0; j<*lt; j++ ) {
         err = GSL_POSINF; a = NULL;
         iter = 0; Nl = 1; Nu = *M;
         while( err > *eps && iter < *max_iter ) {
            a = realloc( a, Nu*sizeof( double ) );
            w = gsl_sum_levin_utrunc_alloc( Nu );
            for (k=Nl; k<=Nu; k++) {
               GSL_SET_COMPLEX( &z, 0.0, -(k*pi)/(*A) );
               a[k-1] = m( z, th )*sin( k*pi*t[j]/(*A) )/k;
            }
            gsl_sum_levin_utrunc_accel( a, Nu, w, &sum_accel, &err );
            gsl_sum_levin_utrunc_free( w );
            iter += 1; Nl += *M; Nu += *M;
         }
         free( a );
         fid[j] = t[j]/(*A) + 2.0*sum_accel/pi;
         fid[j] = (fid[j] < 0.0) ? 0.0 : fid[j];
         fid[j] = (fid[j] > 1.0) ? 1.0 : fid[j];
         fi[j] += fid[j];
         if (iter == *max_iter) { *err_fi = 1; }
      }
      if( *file ) {
         for ( j=0; j<*lt-1; j++ ) {
            fprintf( f, "%g ", fid[j] );
         }
         fprintf( f, "%g\n", fid[*lt-1] );
         fflush( f );
      }
      if( *info && (! (int) fmod( i, 25 ) || i == *N-1) ) {
         fprintf( stderr, "CDF Iterate = %d\n", i );
      }
   }
   if( *file ) {
      fclose( f );
   }
   free( fid ); free( th );
   return;
}
