#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <pthread.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_sum.h>

double m( gsl_complex, double * );

void fourier( int *M, double *A, int *lt, double *t, double *theta,
              int *N, int *ntheta, int *max_iter, int *nthread,
              double *eps, int *info, double *fi, int *err_fi,
              int *file)
{
   int i, j, mm, MM, **errfi, iret[*nthread], NN[*lt];
   double **ffi;
   pthread_t threads[*nthread];
   FILE *f;

   struct pthr { int M, N, ntheta, td, mm, MM, lt, max_iter, info,
                 **err_fi; double A, eps, *theta, *t, **fi; }
               thr[*nthread];
   void *dfeval( void *p )
   {
      int i, j, k, iter, Nl, Nu;
      double pi, fid, sum_accel, err, *th, *a;
      gsl_complex z;
      gsl_sum_levin_utrunc_workspace *w;
      struct pthr *pt = (struct pthr *) p;

      pi = 4.0 * atan( 1.0 );
      th = (double *) malloc( pt->ntheta * sizeof( double ) );
      for( i=pt->mm; i<=pt->MM; i++ ) {
         for( j=0; j<pt->ntheta; j++ ) {
            th[j] = pt->theta[j+i*(pt->ntheta)];
         }
         for( j=0; j<pt->lt; j++ ) {
            err = GSL_POSINF;
            GSL_SET_COMPLEX( &z, -(pt->A)/(2.0*pt->t[j]), 0.0 );
            a = malloc( 1 * sizeof( double ) );
            a[0] = m( z, th )/2.0;
            iter = 0; Nl = 1; Nu = pt->M;
            while( err > pt->eps && iter < pt->max_iter ) {
               a = realloc( a, (Nu+1)*sizeof( double ) );
               w = gsl_sum_levin_utrunc_alloc( Nu+1 );
               for (k=Nl; k<=Nu; k++) {
                  GSL_SET_COMPLEX( &z, -(pt->A)/(2.0*pt->t[j]),
                                       -(k*pi)/pt->t[j] );
                  a[k] = pow( -1, k )*m( z, th );
               }
               gsl_sum_levin_utrunc_accel( a, Nu+1, w, &sum_accel, &err );
               gsl_sum_levin_utrunc_free( w );
               iter += 1; Nl += pt->M; Nu += pt->M;
            }
            free( a );
            fid = (exp( (pt->A)/2.0 )/pt->t[j]) * sum_accel;
            fid = (fid >= 0.0) ? fid : 0.0;
            pt->fi[i][j] = fid;
            if (iter == pt->max_iter) { pt->err_fi[i][j] = 1; }
         }
         if( pt->info && (! (int) fmod( i, 25 ) || i == pt->N-1) ) {
            fprintf( stderr, "Thread = %d, PDF Iterate = %d\n", pt->td, i );
         }
      }
      free( th );
      pthread_exit( NULL );
   }

   errfi = (int **) malloc( *N * sizeof( int * ) );
   for( i=0; i<*N; i++ ) {
      errfi[i] = (int *) calloc( *lt, sizeof( int ) );
   }
   ffi = (double **) malloc( *N * sizeof( double * ) );
   for( i=0; i<*N; i++ ) {
      ffi[i] = (double *) calloc( *lt, sizeof( double ) );
   }
   for( i=0; i<*nthread; i++ ) {
      thr[i].ntheta = *ntheta; thr[i].theta = theta;
      thr[i].lt = *lt; thr[i].t = t;
      thr[i].M = *M; thr[i].A = *A;
      thr[i].N = *N; thr[i].max_iter = *max_iter; thr[i].eps = *eps;
      thr[i].info = *info; thr[i].td=i+1;
      if( *nthread == 1 ) { mm = 0; MM = *N-1; }
      else {
         if( i == 0 ) { mm = 0; MM = *N / *nthread - 1; }
         else if( i == *nthread-1 ) { mm = MM+1; MM = *N-1; }
         else { mm = MM+1; MM = ((i+1) * (*N)) / *nthread - 1; }
      }
      thr[i].mm = mm; thr[i].MM = MM;
      thr[i].fi = ffi; thr[i].err_fi = errfi;
      iret[i] = pthread_create( &threads[i], NULL, dfeval, (void *) &thr[i] );
   }
   for( i=0; i<*nthread; i++ ) {
      pthread_join( threads[i], NULL );
   }

   for( i=0; i<*lt; i++ ) {
      NN[i] = *N;
      for( j=0; j<*N; j++ ) {
         if( !errfi[j][i] ) { fi[i] += ffi[j][i]; }
         else { NN[i]--; }
      }
      if( !NN[i] ) { *err_fi = 1; }
      if( *err_fi == 1 ) {
         for( j=0; j<*N; j++ ) { free( ffi[j] ); free( errfi[j] ); }
         free( ffi ); free( errfi );
         return;
      }
      fi[i] /= NN[i];
   }

   if( *file ) {
      f = fopen( "pd.out", "a" );
      for( i=0; i<*N; i++ ) {
         for( j=0; j<*lt-1; j++ ) {
            fprintf( f, "%g ", ffi[i][j] );
         }
         fprintf( f, "%g\n", ffi[i][*lt-1] );
         fflush( f );
      }
      fclose( f );
   }

   for( i=0; i<*N; i++ ) { free( ffi[i] ); free( errfi[i] ); }
   free( ffi ); free( errfi );
   return;
}
