#
# Read in Inverse Gaussian functions.
#

source("../inv_gauss.r")

#
# Set simulation parameters.
#

n <- 100         # Number of patients.

p10 <- 0.7       # Transitions from 1
p12 <- 1 - p10

r <- 0.01                        # 0 -> 1 waiting time E parameters
l1 <- 10*sqrt(5); l2 <- sqrt(5)  # 1 -> 0 waiting time IG parameters
a1 <- 1000; b1 <- 5              # 1 -> 2 waiting time G parameters

censor <- numeric(n)
visits <- numeric(n)
tottime <- numeric(n)

#
# Simulate censoring times
#

#ctime <- rep(-1, n)

sct <- 50
mct <- 600
ctime <- rnorm(n, mean=mct, sd=sct)

data <- NULL
for (i in 1:n) {
   states <- numeric(); states[1] <- 0;
   time <- numeric(); time[1] <- 0;
   cs <- 0; j <- 1;
   while (1) {
      if (cs == 0) {
         cs <- 1; states[j+1] <- cs;
         time[j+1] <- rexp(1, rate=r); j <- j+1; 
         next;
      }
      else if (cs == 1) {
         x <- rbinom(1, size=1, prob=p10)
         if (x) {
            cs <- 0; states[j+1] <- cs;
            time[j+1] <- rinvgauss(1, a=l1, b=l2); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a1, rate=b1); j <- j+1;
            break;
         }
      }
   }
   out <- numeric(); time <- cumsum(time);
   for (k in 1:(j-1)) {
      out[2*k-1] <- states[k]; out[2*k] <- time[k+1];
      if (ctime[i] >= time[k] && ctime[i] < time[k+1]) {
         out[2*k+1] <- states[k]; out[2*k] <- ctime[i];
         censor[i] <- 1; j <- k+1; break;
      }
   }
   if (!censor[i]) { out[2*j-1] <- states[j]; }
   states <- NULL; time <- NULL; data <- c(data, out);
   tottime[i] <- out[2*j-2]; out <- NULL; visits[i] <- j;
}

data.df <- data.frame(visits=visits, tottime=tottime, censor=censor)
write.table(data.df, file="data_summary", quote=FALSE)
write(data, file="data_complete", ncol=1)
