#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../fourier.r"); source("../sumstats_fi.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_ss"
ntheta <- 6; N <- 2000; nburnin <- 1000; al <- .05;
xlim.up <- 1500
t <- seq(1e-6, xlim.up, 10)

M <- 1000; max.iter <- 10; A <- 20; eps <- 1e-8;

#
# Read the data
#

dat1.df <- read.table("data_summary", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("data_complete")

id <- rep(1:nobs, 2*dat1.df$visits-1)
t01 <- t10 <- t12 <- c0 <- c1 <- NULL

for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) { t01 <- c(t01, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 0 ) { t10 <- c(t10, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 2 ) { t12 <- c(t12, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 0 ) { c0 <- c(c0, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 1 ) { c1 <- c(c1, d[2*j]-it) }
   }
}

ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t10); ip[4] <- ip[3]+length(t12);
ip[5] <- ip[4]+length(c0); ip[6] <- ip[5]+length(c1);
ip[7] <- M; ip[8] <- max.iter;

dp <- c(t01, t10, t12, c0, c1)

lb <- numeric(ntheta)
ub <- c(1, 1, rep(c(10, 0.1), ntheta-2))
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

#
# Find MAP
#

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=MAP$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum)
Theta.ss <- Theta.ss[[1]]

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

save.image()

#
# Set Theta for Fourier inversion approximation and plots
#

Theta <- Theta.ss

#
# Fourier inversion approximation
#

FI <- fourier(N,Theta,t,M,A,max.iter,nthread=3,eps,info=1)
pd <- FI$pd

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, pd, M, A, max.iter, eps,
                  nthread=3, outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
