#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include "../log_density.h"

void transform( double *, int *, double * );
void id_02( double, double, int *, double *, double *, int *, int *,
            double *, double *, double *, double *, int * );

double post( double *theta, int *ip, double *dp )
{

   int i, err=0, dp_id=ip[8]+ip[11], ip_id=9, pp=ip[8]+ip[1]+2;
   double pdf, sf1, sf2, tmp, *ttheta, *p=theta, *t, *sp;

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      for( i=0; i<ip[1]; i++ ) {
         ttheta[i] = theta[i];
      }
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {
/* Boundaries for prior 1. */

      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[8]+i] ) {
            return( HUGE_VAL );
         }
      }
      for( i=0; i<2; i++ ) {
         if( p[i] >= dp[ip[8]+ip[1]+i] ) {
            return( HUGE_VAL );
         }
      }
   }

/* Calculate likelihood */

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[1] );
      tmp += pdf;
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      pdf = log_invgauss_pdf( dp[i], p[2], p[3] );
      tmp += pdf + log(1.0 - p[0]);
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      pdf = log_gamma_pdf( dp[i], p[4], p[5] );
      tmp += pdf + log(p[0]);
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      sf1 = log_exponential_sf( dp[i], p[1] );
      tmp += sf1;
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      invgauss_sf( dp[i], p[2], p[3], &sf1 );
      sf1 *= (1.0 - p[0]);
      gamma_sf( dp[i], p[4], p[5], &sf2 );
      tmp += log( sf1 + p[0]*sf2 );
   }

/* Incomplete Data */
   t = (double *) malloc( ip[ip_id] * sizeof( double ) );
   for( i=ip[6]; i<ip[7]; i++ ) {
      t[i-ip[6]] = dp[i];
   }
   sp = (double *) calloc( ip[ip_id], sizeof( double ) );

   id_02( dp[dp_id], dp[dp_id+1], &ip[ip_id], t, p, &ip[1],
          &ip[ip_id+1], &dp[dp_id+2], &dp[dp_id+3], &dp[dp_id+4],
          sp, &err );

   free( t );
   if( err ) {
      free( sp );
      if( ip[0] ) {
         free( ttheta );
      }
      return( HUGE_VAL );
   }
   for( i=ip[7]; i<ip[8]; i++ ) {
      tmp += dp[i] * log( sp[i-ip[7]] );
   }
   free( sp );

/* Calculate prior 1 */
   for( i=0; i<4; i++ ) {
      pdf = log_gamma_pdf( p[i+2], dp[pp+2*i], dp[pp+2*i+1] );
      tmp += pdf;
   }

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}
