#define PI 3.141592653589793116
#define SQRT2 1.41421356237309514548
#define LOG2 0.69314718055994528623

#include <math.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_sf_erf.h>
#include <gsl/gsl_sf_result.h>
#include <gsl/gsl_errno.h>

double log_exponential_pdf( double x, double l )
{
   return( log( l ) - l * x );
}

double log_gamma_pdf( double x, double a, double b )
{
   double lga;
   lga = (a == 1.0) ? 0.0 : gsl_sf_lngamma( a );
   return( a * log( b ) - lga + (a - 1) * log( x ) - b * x );
}

double log_invgauss_pdf( double x, double a, double b )
{
   return( log( a ) - log( 2 * PI ) / 2 - 3 * log( x ) / 2 -
           ( pow( a, 2 ) / x + pow( b, 2 ) * x ) / 2 + a * b );
}

double log_gauss_pdf( double x, double a, double b )
{
   return( -pow(x - a, 2) / (2*b) - log( 2 * PI * b ) / 2 );
}

double log_exponential_sf( double x, double l )
{
   return( -l * x );
}

int gamma_sf( double x, double a, double b, double *sf )
{
   int status=0;
   gsl_error_handler_t *gsl_errh;
   gsl_sf_result sv;

   gsl_errh = gsl_set_error_handler_off();
   status = gsl_sf_gamma_inc_Q_e( a, b*x, &sv );
   gsl_set_error_handler( gsl_errh );

   if( status ) { return( status ); }
   *sf = sv.val;
   return( status );
}

int invgauss_sf( double x, double a, double b, double *sf )
{
   int status=0;
   double sx=sqrt(x);
   gsl_error_handler_t *gsl_errh;
   gsl_sf_result sv;

   gsl_errh = gsl_set_error_handler_off();
   status = gsl_sf_erf_Q_e( b * sx - a / sx, &sv );
   if( status ) { gsl_set_error_handler( gsl_errh ); return( status ); }
   *sf = sv.val;
   status = gsl_sf_log_erfc_e( (a / sx + b * sx) / SQRT2, &sv );
   gsl_set_error_handler( gsl_errh );

   if( status ) { return( status ); }
   *sf -= exp( sv.val - LOG2 + 2 * a * b ); 
   return( status );
}
