#
# Read in Inverse Gaussian functions.
#

source("../inv_gauss.r")

#
# Set simulation parameters.
#

n <- 100         # Number of patients.

p01 <- 0.6       # Transition from 0

p35 <- 0.7       # Transition from 3

p45 <- 0.5       # Transition from 4

r01 <- 1/8                         # 0 -> 1 waiting time E parameter
r02 <- 10/3                        # 0 -> 2 waiting time E parameter
l13.1 <- 5; l13.2 <- 4/5           # 1 -> 3 waiting time IG parameters
l24.1 <- 2; l24.2 <- 4             # 2 -> 4 waiting time IG parameters
a31 <- 5; b31 <- 4                 # 3 -> 1 waiting time G parameters
a42 <- 7; b42 <- 3                 # 4 -> 2 waiting time G parameters
r35 <- 2                           # 3 -> 5 waiting time E parameter
r45 <- 1/4                         # 4 -> 5 waiting time E parameter

censor <- numeric(n)
visits <- numeric(n)
tottime <- numeric(n)

#
# Simulate censoring times
#

#ctime <- rep(-1, n)

sct <- 1
mct <- 22
ctime <- rnorm(n, mean=mct, sd=sct)

data <- NULL
for (i in 1:n) {
   states <- numeric(); states[1] <- 0;
   time <- numeric(); time[1] <- 0;
   cs <- 0; j <- 1;
   while (1) {
      if (cs == 0) {
         x <- rbinom(1, size=1, prob=p01)
         if (x) {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r01); j <- j+1; 
            next;
         } else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r02); j <- j+1;
            next;
         }
      } else if (cs == 1) {
         cs <- 3; states[j+1] <- cs;
         time[j+1] <- rinvgauss(1, a=l13.1, b=l13.2); j <- j+1;
         next;
      } else if (cs == 2) {
         cs <- 4; states[j+1] <- cs;
         time[j+1] <- rinvgauss(1, a=l24.1, b=l24.2); j <- j+1;
         next;
      } else if (cs == 3) {
         x <- rbinom(1, size=1, prob=p35)
         if (x) {
            cs <- 5; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r35); j <- j+1;
            break;
         } else {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a31, rate=b31); j <- j+1;
            next;
         }
      } else if (cs == 4) {
         x <- rbinom(1, size=1, prob=p45)
         if (x) {
            cs <- 5; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r45); j <- j+1;
            break;
         } else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rgamma(1, shape=a42, rate=b42); j <- j+1;
            next;
         }
      }
   }
   out <- numeric(); time <- cumsum(time);
   for (k in 1:(j-1)) {
      out[2*k-1] <- states[k]; out[2*k] <- time[k+1];
      if (ctime[i] >= time[k] && ctime[i] < time[k+1]) {
         out[2*k+1] <- states[k]; out[2*k] <- ctime[i];
         censor[i] <- 1; j <- k+1; break;
      }
   }
   if (!censor[i]) { out[2*j-1] <- states[j]; }
   states <- NULL; time <- NULL; data <- c(data, out);
   tottime[i] <- out[2*j-2]; out <- NULL; visits[i] <- j;
}

data.df <- data.frame(visits=visits, tottime=tottime, censor=censor)
write.table(data.df, file="data_summary", quote=FALSE)
write(data, file="data_complete", ncol=1)
