#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_psi.h>
#include "../log_density.h"

void transform( double *, int *, double * );
void id_03( double, double, int *, double *, double *, int *, int *,
            double *, double *, double *, double *, int * );
void id_25( double, double, int *, double *, double *, int *, int *,
            double *, double *, double *, double *, int * );

double post( double *theta, int *ip, double *dp )
{

   int i, err=0, dp_id=ip[16]+ip[22], ip_id=17;
   double *id_p, *t, *sp;
   double pdf, sf1, sf2, tmp, wk, *ttheta, *p=theta;

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      for( i=0; i<ip[1]; i++ ) {
         ttheta[i] = theta[i];
      }
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {
/* Boundaries for prior */

      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[16]+i] ) {
            return( HUGE_VAL );
         }
      }
      for( i=0; i<ip[1]; i++ ) {
         if( p[i] >= dp[ip[16]+ip[1]+i] ) {
            return( HUGE_VAL );
         }
      }
   }

/* Calculate likelihood */

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[3] );
      tmp += pdf + log( p[0] );
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      wk = sqrt( p[4] / p[5] );
      pdf = log_invgauss_pdf( dp[i], p[4] * wk, wk );
      tmp += pdf;
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      wk = p[6] / p[7];
      pdf = log_gamma_pdf( dp[i], p[6] * wk, wk );
      tmp += pdf + log( p[1] );
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[8] );
      tmp += pdf + log( 1.0 - p[1] );
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[9] );
      tmp += pdf + log( 1.0 - p[0] );
   }
   for( i=ip[6]; i<ip[7]; i++ ) {
      wk = sqrt( p[10] / p[11] );
      pdf = log_invgauss_pdf( dp[i], p[10] * wk, wk );
      tmp += pdf;
   }
   for( i=ip[7]; i<ip[8]; i++ ) {
      wk = p[12] / p[13];
      pdf = log_gamma_pdf( dp[i], p[12] * wk, wk );
      tmp += pdf + log( p[2] );
   }
   for( i=ip[8]; i<ip[9]; i++ ) {
      pdf = log_exponential_pdf( dp[i], p[14] );
      tmp += pdf + log( 1.0 - p[2] );
   }
   for( i=ip[9]; i<ip[10]; i++ ) {
      sf1 = log_exponential_sf( dp[i], p[3] );
      sf1 = p[0] * exp( sf1 );
      sf2 = log_exponential_sf( dp[i], p[9] );
      sf2 = (1.0 - p[0]) * exp( sf2 );
      tmp += log( sf1 + sf2 );
   }
   for( i=ip[10]; i<ip[11]; i++ ) {
      wk = sqrt( p[4] / p[5] );
      invgauss_sf( dp[i], p[4] * wk, wk, &sf1 );
      tmp += log( sf1 );
   }
   for( i=ip[11]; i<ip[12]; i++ ) {
      wk = p[6] / p[7];
      gamma_sf( dp[i], p[6] * wk, wk, &sf1 );
      sf1 *= p[1];
      sf2 = log_exponential_sf( dp[i], p[8] );
      sf2 = (1.0 - p[1]) * exp( sf2 );
      tmp += log( sf1 + sf2 );
   }

/* Incomplete Data */
   id_p = (double *) malloc( ip[ip_id] * sizeof( double ) );
   id_p[0] = p[1];
   id_p[1] = p[3];
   id_p[3] = sqrt( p[4] / p[5] );
   id_p[2] = p[4] * id_p[3];
   id_p[5] = p[6] / p[7];
   id_p[4] = p[6] * id_p[5];

   t = (double *) malloc( ip[ip_id+1] * sizeof( double ) );
   for( i=ip[12]; i<ip[13]; i++ ) {
      t[i-ip[12]] = dp[i];
   }
   sp = (double *) calloc( ip[ip_id+1], sizeof( double ) );

   id_03( dp[dp_id], dp[dp_id+1], &ip[ip_id+1], t, id_p, &ip[ip_id],
          &ip[ip_id+4], &dp[dp_id+2], &dp[dp_id+3], &dp[dp_id+4],
          sp, &err );

   free( id_p ); free( t );
   if( err ) {
      free( sp );
      if( ip[0] ) {
         free( ttheta );
      }
      return( HUGE_VAL );
   }
   for( i=ip[13]; i<ip[14]; i++ ) {
      tmp += log( p[0] ) + dp[i] * log( sp[i-ip[13]] );
   }
   free( sp );

   id_p = (double *) malloc( ip[ip_id+2] * sizeof( double ) );
   id_p[0] = p[2];
   id_p[2] = sqrt( p[10] / p[11] );
   id_p[1] = p[10] * id_p[2];
   id_p[4] = p[12] / p[13];
   id_p[3] = p[12] * id_p[4];
   id_p[5] = p[14];

   t = (double *) malloc( ip[ip_id+3] * sizeof( double ) );
   for( i=ip[14]; i<ip[15]; i++ ) {
      t[i-ip[14]] = dp[i];
   }
   sp = (double *) calloc( ip[ip_id+3], sizeof( double ) );

   id_25( dp[dp_id], dp[dp_id+1], &ip[ip_id+3], t, id_p, &ip[ip_id+2],
          &ip[ip_id+4], &dp[dp_id+2], &dp[dp_id+3], &dp[dp_id+4],
          sp, &err );

   free( id_p ); free( t );
   if( err ) {
      free( sp );
      if( ip[0] ) {
         free( ttheta );
      }
      return( HUGE_VAL );
   }
   for( i=ip[15]; i<ip[16]; i++ ) {
      tmp += dp[i] * log( sp[i-ip[15]] );
   }
   free( sp );

/* Jeffrey's prior */

/* Exponential branches */

   tmp += -log( p[3] ) - log( p[8] ) - log( p[9] ) - log( p[14] );

/* Gamma branches */

   wk = pow( p[6], 2 ) / p[7];
   tmp += log( wk * gsl_sf_psi_n( 1, wk ) - 1.0 ) / 2.0 +
          log( p[6] ) - 2.0 * log( p[7] );
   wk = pow( p[12], 2 ) / p[13];
   tmp += log( wk * gsl_sf_psi_n( 1, wk ) - 1.0 ) / 2.0 +
          log( p[12] ) - 2.0 * log( p[13] );

/* Inverse Gaussian branches */

   tmp += -3.0 * (log( p[5] ) + log( p[11] )) / 2.0;

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}
