#
# Source R functions
#

source("../transform.r");
source("../optimize.r"); source("../slice.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../mcvar.r"); source("../plots.r");

#
# Set-up
#

outsum <- "summary_ss"; outpd <- "mc_pd"; outcdf <- "mc_cdf";
outhf <- "mc_hf"; outmc <- "mc_ss";
ntheta <- 15; N <- 15000; nburnin <- 1000; al <- .05;
m <- 1e-6; M <- 1e6;
xlim.up <- 50
t <- seq(m, xlim.up, length=100)
bch <- 1; nb <- 30;

max.iter <- 1000; epsrel <- 1e-7; epsabs <- 0; max.abserr <- 1e-4;
ntheta03 <- 6; ntheta25 <- 6;

#
# Read the data
#

dat1.df <- read.table("../mmr_sim/data_summary1", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("../mmr_sim/data_complete1")

id <- rep(1:nobs, 2*dat1.df$visits-1)
t01 <- t02 <- t03 <- t13 <- t24 <- t25 <- t31 <- t35 <- t42 <- t45 <-
c0 <- c1 <- c2 <- NULL

for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) { t01 <- c(t01, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 2 ) { t02 <- c(t02, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 3 ) { t03 <- c(t03, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 3 ) { t13 <- c(t13, d[2*j]-it) }
      if( d[2*j-1] == 2 && d[2*j+1] == 4 ) { t24 <- c(t24, d[2*j]-it) }
      if( d[2*j-1] == 2 && d[2*j+1] == 5 ) { t25 <- c(t25, d[2*j]-it) }
      if( d[2*j-1] == 3 && d[2*j+1] == 1 ) { t31 <- c(t31, d[2*j]-it) }
      if( d[2*j-1] == 3 && d[2*j+1] == 5 ) { t35 <- c(t35, d[2*j]-it) }
      if( d[2*j-1] == 4 && d[2*j+1] == 2 ) { t42 <- c(t42, d[2*j]-it) }
      if( d[2*j-1] == 4 && d[2*j+1] == 5 ) { t45 <- c(t45, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 0 ) { c0 <- c(c0, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 1 ) { c1 <- c(c1, d[2*j]-it) }
      if( d[2*j-1] == 3 && d[2*j+1] == 3 ) { c2 <- c(c2, d[2*j]-it) }
   }
}

t03 <- table(sort(t03))
d03 <- as.numeric(t03)
t03 <- as.numeric(names(t03))

t25 <- table(sort(t25))
d25 <- as.numeric(t25)
t25 <- as.numeric(names(t25))

ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t13); ip[4] <- ip[3]+length(t31);
ip[5] <- ip[4]+length(t35); ip[6] <- ip[5]+length(t02);
ip[7] <- ip[6]+length(t24); ip[8] <- ip[7]+length(t42);
ip[9] <- ip[8]+length(t45);
ip[10] <- ip[9]+length(c0); ip[11] <- ip[10]+length(c1);
ip[12] <- ip[11]+length(c2);
ip[13] <- ip[12]+length(t03); ip[14] <- ip[13]+length(d03);
ip[15] <- ip[14]+length(t25); ip[16] <- ip[15]+length(d25);
ip[17] <- ntheta03; ip[18] <- length(t03);
ip[19] <- ntheta25; ip[20] <- length(t25);
ip[21] <- max.iter

dp <- c(t01,t13,t31,t35,t02,t24,t42,t45,c0,c1,c2,t03,d03,t25,d25)

lb <- numeric(ntheta)
ub <- c(rep(1,3),  rep(10^6,ntheta-3))
prior.p <- c(lb, ub)
ip[22] <- length(prior.p)

dp <- c(dp, prior.p)

id <- c(m, M, epsrel, epsabs, max.abserr)
dp <- c(dp, id)

#
# Find MAP
#

system("cp post_map.c post.c")

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum, id=1)

system("rm post.c")

system("cp post_ss.c post.c")

SS <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum, id=1)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=SS$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, id=1, outfile=outsum,
                  file=TRUE)
Theta.ss <- as.matrix(read.table("theta.out"))

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

system("rm post.c")

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

if (nburnin > 0) {
   Theta.ss.burnin[,6] <- sqrt(Theta.ss.burnin[,5] / Theta.ss.burnin[,6])
   Theta.ss.burnin[,5] <- Theta.ss.burnin[,5] * Theta.ss.burnin[,6]
   Theta.ss.burnin[,8] <- Theta.ss.burnin[,7] / Theta.ss.burnin[,8]
   Theta.ss.burnin[,7] <- Theta.ss.burnin[,7] * Theta.ss.burnin[,8]
   Theta.ss.burnin[,12] <- sqrt(Theta.ss.burnin[,11] / Theta.ss.burnin[,12])
   Theta.ss.burnin[,11] <- Theta.ss.burnin[,11] * Theta.ss.burnin[,12]
   Theta.ss.burnin[,14] <- Theta.ss.burnin[,13] / Theta.ss.burnin[,14]
   Theta.ss.burnin[,13] <- Theta.ss.burnin[,13] * Theta.ss.burnin[,14]
}

Theta.ss[,6] <- sqrt(Theta.ss[,5] / Theta.ss[,6])
Theta.ss[,5] <- Theta.ss[,5] * Theta.ss[,6]
Theta.ss[,8] <- Theta.ss[,7] / Theta.ss[,8]
Theta.ss[,7] <- Theta.ss[,7] * Theta.ss[,8]
Theta.ss[,12] <- sqrt(Theta.ss[,11] / Theta.ss[,12])
Theta.ss[,11] <- Theta.ss[,11] * Theta.ss[,12]
Theta.ss[,14] <- Theta.ss[,13] / Theta.ss[,14]
Theta.ss[,13] <- Theta.ss[,13] * Theta.ss[,14]

Theta <- Theta.ss

save.image()

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3, file=TRUE)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Monte Carlo summaries
#

mc.pd <- read.table("pd.out")
pd.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(mc.pd[,i])$var) })

z.cv <- qnorm(0.975)
pd.lb <- pd - z.cv * pd.mcsd
pd.lb <- ifelse(pd.lb < 0, 0, pd.lb)
pd.ub <- pd + z.cv * pd.mcsd

write.table(cbind(pd.lb, pd, pd.ub), outpd, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

out <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, numeric(length(t)),
                task=1, outfile=outsum)
mu <- out$vdk; var <- out$vddk + (mu - mean(mu))^2;

save.image()

out <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd, task=2,
                nthread=3, outfile=outsum, file=TRUE)
cdf <- read.table("cdf.out")

save.image()

mu.mcmu <- mean(mu); var.mcmu <- mean(var+(mu-mu.mcmu)^2);
cdf.mcmu <- apply(cdf, 2, mean)

mu.mcsd <- sqrt(mcvar(mu)$var)
var.mcsd <- sqrt(mcvar(var+(mu-mu.mcmu)^2)$var)
cdf.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(cdf[,i])$var) })

cdf.lb <- cdf.mcmu - z.cv * cdf.mcsd
cdf.lb <- ifelse(cdf.lb < 0, 0, cdf.lb)
cdf.ub <- cdf.mcmu + z.cv * cdf.mcsd
cdf.ub <- ifelse(cdf.ub > 1, 1, cdf.ub)

write.table(cbind(cdf.lb, cdf.mcmu, cdf.ub), outcdf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd, cdf=cdf.mcmu,
                  task=3, nthread=3, outfile=outsum)
q <- stats$q
xlim.up <- max(xlim.up, q[length(q)])

save.image()

sf <- 1 - cdf.mcmu; hf <- pd / sf;
hf.est <- t(sapply(1:nb,
          function (i) {
             sb <- N/nb
             iss <- (1+(i-1)*sb):(i*sb)
             pd.t <- apply(mc.pd[iss,],2,mean)
             cdf.t <- apply(cdf[iss,],2,mean)
             pd.t / (1 - cdf.t)
          }))
hf.mcmu <- apply(hf.est, 2, mean)
hf.mcsd <- sapply(1:length(t),
                   function (i) {
                      if (bch) {
                         hf.mcvar <- mcvar(hf.est[,i], b=nb, method="batch")
                         c(hf.mcvar$acf, sqrt(hf.mcvar$var))
                      } else {
                         hf.mcvar <- mcvar(hf.est[,i])
                         c(hf.mcvar$ess, sqrt(hf.mcvar$var))
                      }
                   })
if (bch) { hf.acf <- hf.mcsd[1,] } else { hf.ess <- hf.mcsd[1,] }
hf.mcsd <- hf.mcsd[2,]

hf.lb <- hf.mcmu - z.cv * hf.mcsd
hf.lb <- ifelse(hf.lb < 0, 0, hf.lb)
hf.ub <- hf.mcmu + z.cv * hf.mcsd

write.table(cbind(hf.lb, hf.mcmu, hf, hf.ub), outhf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

q.est <- t(sapply(1:nb,
         function (i) {
            sb <- N/nb
            iss <- (1+(i-1)*sb):(i*sb)
            cdf.t <- apply(cdf[iss,],2,mean)
            if (i == 1) {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, clean=F)$q
            } else if (i < nb) {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, compile=F, clean=F)$q
            } else {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, compile=F)$q
            }
         }))
rm(mc.pd, cdf)

save.image()

q.mcmu <- apply(q.est, 2, mean)
q.mcsd <- sapply(1:length(q),
                  function (i) {
                     if (bch) {
                        q.mcvar <- mcvar(q.est[,i], b=nb, method="batch")
                        c(q.mcvar$acf, sqrt(q.mcvar$var))
                     } else {
                        q.mcvar <- mcvar(q.est[,i])
                        c(q.mcvar$ess, sqrt(q.mcvar$var))
                     }
                  })
if (bch) { q.acf <- q.mcsd[1,] } else { q.ess <- q.mcsd[1,] }
q.mcsd <- q.mcsd[2,]

q.lb <- q.mcmu - z.cv * q.mcsd
q.lb <- ifelse(q.lb < 0, 0, q.lb)
q.ub <- q.mcmu + z.cv * q.mcsd

save.image()

write("Confidence Interval for PD Mean:\n", file=outmc)
write(c(mu.mcmu - z.cv * mu.mcsd, mu.mcmu, mu.mcmu + z.cv * mu.mcsd),
      file=outmc, ncol=3, append=TRUE)
write("\nConfidence Interval for PD Standard Deviation:\n",
      file=outmc, append=TRUE)
write(c(sqrt(var.mcmu - z.cv * var.mcsd), sqrt(var.mcmu),
        sqrt(var.mcmu + z.cv * var.mcsd)), file=outmc,
      ncol=3, append=TRUE)
write("\nConfidence Intervals for PD Quantiles:\n", file=outmc,
      append=TRUE)
write(t(cbind(q.lb, q.mcmu, q, q.ub)), file=outmc, ncol=4, append=TRUE)

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
