#
# Read in Inverse Gaussian functions.
#

source("../inv_gauss.r")

#
# Set simulation parameters.
#

n <- 100         # Number of patients.

p10 <- 0.4

l1 <- 30; l2 <- 3;   # 0 -> 1 waiting time IG parameters
phi2 <- 1/3          # 1 -> 0 waiting time G parameters
r <- 0.2             # 1 -> 2 waiting time E parameters

censor <- numeric(n)
visits <- numeric(n)
tottime <- numeric(n)

#
# Simulate censoring times
#

ctime <- rep(-1, n)

#sct <- 1/3
#mct <- 6.7
#ctime <- rnorm(n, mean=mct, sd=sct)

#
# Generate patient covariates
#

b.p01 <- c(0, 1)

b.m1 <- c(1.85, .3, -.4)
b.phi1 <- c(-.5, .4)
b.m2 <- c(-.5, .75, .4)

x1 <- seq(-1, 1, length=n)
x2 <- runif(n, min=-1)
X <- cbind(rep(1, n), x1, x2)

lm.p01 <- X[,1:2] %*% b.p01
p01 <- exp(lm.p01)/(1+exp(lm.p01))

lm.m1 <- X %*% b.m1
m1 <- exp(lm.m1)
lm.phi1 <- X[,c(1,3)] %*% b.phi1
phi1 <- exp(lm.phi1)
lm.m2 <- X %*% b.m2
m2 <- exp(lm.m2)

data <- NULL
for (i in 1:n) {
   states <- numeric(); states[1] <- 0;
   time <- numeric(); time[1] <- 0;
   cs <- 0; j <- 1;
   while (1) {
      if (cs == 0) {
         x <- rbinom(1, size=1, prob=p01[i])
         if (x) {
            cs <- 1; states[j+1] <- cs;
            time[j+1] <- rinvgauss(1, a=l1, b=l2); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            a1 <- 1/phi1[i]; b1 <- a1/m1[i];
            time[j+1] <- rgamma(1, shape=a1, rate=b1); j <- j+1;
            break;
         }
      }
      else if (cs == 1) {
         x <- rbinom(1, size=1, prob=p10)
         if (x) {
            cs <- 0; states[j+1] <- cs;
            a2 <- 1/phi2; b2 <- a2/m2[i];
            time[j+1] <- rgamma(1, shape=a2, rate=b2); j <- j+1;
            next;
         }
         else {
            cs <- 2; states[j+1] <- cs;
            time[j+1] <- rexp(1, rate=r); j <- j+1;
            break;
         }
      }
   }
   out <- numeric(); time <- cumsum(time);
   for (k in 1:(j-1)) {
      out[2*k-1] <- states[k]; out[2*k] <- time[k+1];
      if (ctime[i] >= time[k] && ctime[i] < time[k+1]) {
         out[2*k+1] <- states[k]; out[2*k] <- ctime[i];
         censor[i] <- 1; j <- k+1; break;
      }
   }
   if (!censor[i]) { out[2*j-1] <- states[j]; }
   states <- NULL; time <- NULL; data <- c(data, out);
   tottime[i] <- out[2*j-2]; out <- NULL; visits[i] <- j;
}

data.df <- data.frame(visits=visits, tottime=tottime, censor=censor,
                      x1=x1, x2=x2)
write.table(data.df, file="data_summary", quote=FALSE)
write(data, file="data_complete", ncol=1)
