#
# Source R functions
#

source("../transform.r"); source("../optimize.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_mle"
ntheta <- 9; N <- 1;
m <- 1e-6; M <- 1e6;
xlim.up <- 15
t <- seq(m, xlim.up, 0.25)

#
# Read the data
#

dat1.df <- read.table("data_summary", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("data_complete")

id <- rep(1:nobs, 2*dat1.df$visits-1)
t02 <- t01 <- t10 <- t12 <- c0 <- c1 <- NULL

for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) { t01 <- c(t01, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 2 ) { t02 <- c(t02, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 0 ) { t10 <- c(t10, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 2 ) { t12 <- c(t12, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 0 ) { c0 <- c(c0, d[2*j]-it) }
      if( d[2*j-1] == 1 && d[2*j+1] == 1 ) { c1 <- c(c1, d[2*j]-it) }
   }
}
      
ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t02); ip[4] <- ip[3]+length(t10);
ip[5] <- ip[4]+length(t12); ip[6] <- ip[5]+length(c0);
ip[7] <- ip[6]+length(c1)
dp <- c(t01, t02, t10, t12, c0, c1)

lb <- numeric(ntheta)
ub <- c(1, 1)
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

#
# Find MLE
#

MLE <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

Theta <- MLE$theta

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd,
                  outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
