#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../glm.r"); source("../fourier.r");
source("../sumstats_fi.r"); source("../mcvar.r");
source("../plots.r");

#
# Set-up
#

outsum <- "summary_ss"
ntheta <- 15; N <- 2000; nburnin <- 1000; al <- .05;
xlim.up <- 50
t <- seq(1e-6, xlim.up, 0.4)

M <- 1000; max.iter <- 10; A <- 20; eps <- 1e-8;

#
# Read the data
#

dat1.df <- read.table("data_summary", header=TRUE)
nobs <- dim(dat1.df)[1]
dat2 <- scan("data_complete")

# Count transitions / collect waiting times

id <- rep(1:nobs, 2*dat1.df$visits-1)
t02 <- t01 <- t10 <- t12 <- c0 <- c1 <- NULL
cv01 <- cv02 <- cv10 <- cv0 <- cv1 <- NULL
for( i in 1:nobs ) {
   d <- dat2[id==i]
   for( j in 1:(dat1.df$visits[i]-1) ) {
      if( j == 1 ) { it <- 0 }
      else { it <- d[2*j-2] }
      if( d[2*j-1] == 0 && d[2*j+1] == 1 ) {
         t01 <- c(t01, d[2*j]-it); cv01 <- c(cv01, i);
      }
      if( d[2*j-1] == 0 && d[2*j+1] == 2 ) {
         t02 <- c(t02, d[2*j]-it); cv02 <- c(cv02, i);
      }
      if( d[2*j-1] == 1 && d[2*j+1] == 0 ) {
         t10 <- c(t10, d[2*j]-it); cv10 <- c(cv10, i);
      }
      if( d[2*j-1] == 1 && d[2*j+1] == 2 ) { t12 <- c(t12, d[2*j]-it) }
      if( d[2*j-1] == 0 && d[2*j+1] == 0 ) {
         c0 <- c(c0, d[2*j]-it); cv0 <- c(cv0, i);
      }
      if( d[2*j-1] == 1 && d[2*j+1] == 1 ) {
         c1 <- c(c1, d[2*j]-it); cv1 <- c(cv1, i);
      }
   }
}

cp <- c(cv01, cv02, cv10, cv0, cv1) - 1
      
ip <- numeric(); ip[1] <- ntheta; ip[2] <- length(t01);
ip[3] <- ip[2]+length(t02); ip[4] <- ip[3]+length(t10);
ip[5] <- ip[4]+length(t12); ip[6] <- ip[5]+length(c0);
ip[7] <- ip[6]+length(c1); ip <- c(ip, nobs, cp);
dp <- c(t01, t02, t10, t12, c0, c1, dat1.df$x1, dat1.df$x2)

lb <- c(-1e6, -1e6, 0, 0, 0, rep(-1e6, 8), 0, 0)
ub <- c(1e6, 1e6, 1, rep(1e6, ntheta-3))
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

save.image()

#
# Find MAP
#

system("cp post_map.c post.c")

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

system("rm post.c")

system("cp post_ss.c post.c")

sp <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

system("rm post.c")

save.image()

#
# Slice sampling from posterior
#

system("cp post_ss.c post.c")

Theta.ss <- slice(N, ntheta, ip, dp, start=sp$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum,
                  file=TRUE)
Theta.ss <- as.matrix(read.table("theta.out"))

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

save.image()

system("rm post.c")

#
# Set Theta for Fourier inversion approximation and plots
#

if (nburnin > 0) {
   wk <- 1/sqrt(Theta.ss.burnin[,5])
   Theta.ss.burnin[,5] <- wk/Theta.ss.burnin[,4]
   Theta.ss.burnin[,4] <- wk
   Theta.ss.burnin[,15] <- 1/Theta.ss.burnin[,15]
}
wk <- 1/sqrt(Theta.ss[,5])
Theta.ss[,5] <- wk/Theta.ss[,4]
Theta.ss[,4] <- wk
Theta.ss[,15] <- 1/Theta.ss[,15]

Theta <- Theta.ss

save.image()

#
# Fourier inversion approximation
#

np <- 2; x <- c(-0.25, .25, 0, 0);
pd <- sf <- hf <- mu <- list()
ptm1 <- ptm2 <- list()
for (i in 1:np) {
   q <- numeric(ntheta-6)
   Theta.cv <- t(sapply(1:N, function (j) {
                 q[2:4] <- Theta[j, 3:5]
                 q[8:9] <- Theta[j, (ntheta-1):ntheta]
                 if (j == 1) {
                    cl <- T; cn <- F;
                 } else if (j < N) {
                    cl <- F; cn <- F;
                 } else {
                    cl <- F; cn <- T;
                 }
                 for (k in 1:3) {
                    if (k != 1) {
                       cl <- F
                    }
                    if (k != 3) {
                       cn <- F
                    } else if (j == N) {
                       cn <- T
                    }
                    q <- glm(Theta[j,], q, np, i-1, x, k, compile=cl,
                             clean=cn) 
                 }
                 q 
               }))
   wk <- 1/Theta.cv[,6]
   Theta.cv[,6] <- wk/Theta.cv[,5]
   Theta.cv[,5] <- wk
   wk <- 1/Theta.cv[,8]
   Theta.cv[,8] <- wk/Theta.cv[,7]
   Theta.cv[,7] <- wk

   ptm1[[i]] <- proc.time()
   FI <- fourier(N,Theta.cv,t,M,A,max.iter,nthread=3,eps,info=1)
   ptm1[[i]] <- proc.time()-ptm1[[i]]
   pd[[i]] <- FI$pd

   save.image()

   ptm2[[i]] <- proc.time()
   stats <- sumstats(N, Theta.cv, t, pd[[i]], M, A, max.iter, eps,
                     nthread=3, outfile=outsum)
   ptm2[[i]] <- proc.time()-ptm2[[i]]
   sf[[i]] <- stats$sf; hf[[i]] <- stats$hf; mu[[i]] <- stats$vdk;

   save.image()
}

diff.mu <- mu[[2]] - mu[[1]]; mcmu.diff.mu <- mean(diff.mu);
mcsd.diff.mu <- sqrt(mcvar(diff.mu)$var)
z.cv <- qnorm(0.975)
mcmu.diff.mu
mcmu.diff.mu - z.cv * mcsd.diff.mu
mcmu.diff.mu + z.cv * mcsd.diff.mu

save.image()

#
# Plots
#

ci.95 <- apply(Theta, 2, quantile, p=c(.025, .975))
xx <- c(0, 0.1, 0.25, 0.4, 0.5, 0.65, 0.75, 0.85, 1, 1.1, 1.25,
        1.35, 1.45, 1.6, 1.75)
sim.v <- c(0, 1, 0.4, 30, 3, 1.85, .3, -.4, -.5, .4, -.5, .75, .4,
           1/3, 0.2)
dv <- c(rep(0, 3), 29, rep(0, ntheta-4))
y.lb <- min(ci.95)-5; y.ub <- max(ci.95)-max(dv);

postscript("par.ps", width=8, height=8, paper="special",
           horizontal=FALSE)
par(pty="s", oma=c(0,0,1,1), mar=c(5,5,0,0))
plot(0:1, 0:1, xlim=c(min(xx)-.1, max(xx)+.1), ylim=c(y.lb, y.ub),
     type="n", axes=F, xlab="Parameters", ylab="Parameter Values",
     cex.lab=1.5)
axis(2)
box()
ii <- c(1:2, 6:13)
for (i in 1:ntheta) {
   if (i %in% ii) {
      cc <- "red"
   } else {
      cc <- "green"
   }
   segments(xx[i], ci.95[1,i]-dv[i], xx[i], ci.95[2,i]-dv[i], col=cc, lwd=2)
   segments(xx[i]-.03, ci.95[1,i]-dv[i], xx[i]+.03, ci.95[1,i]-dv[i],
            col=cc, lwd=2)
   segments(xx[i]-.03, ci.95[2,i]-dv[i], xx[i]+.03, ci.95[2,i]-dv[i],
            col=cc, lwd=2)
   points(xx[i], sim.v[i]-dv[i], col="magenta", lwd=2)
   points(xx[i], MAP$theta[i]-dv[i], pch=2, col="blue", lwd=2)
}
arrows(xx[1]-.03, -2, xx[2]+.03, -2, code=3, length=.1)
text((xx[1]+xx[2])/2, -2.3, expression(p['01']), cex=1.5)
text(xx[3], -2.3, expression(p['10']), cex=1.5)
text(xx[4], -6, expression(lambda[1]-29), cex=1.5)
text(xx[5], -2.3, expression(lambda[2]), cex=1.5)
arrows(xx[6]-.03, -2, xx[8]+.03, -2, code=3, length=.1)
text((xx[6]+xx[8])/2, -2.3, expression(m[1]), cex=1.5)
arrows(xx[9]-.03, -2, xx[10]+.03, -2, code=3, length=.1)
text((xx[9]+xx[10])/2, -2.3, expression(phi[1]), cex=1.5)
arrows(xx[11]-.03, -2, xx[13]+.03, -2, code=3, length=.1)
text((xx[11]+xx[13])/2, -2.3, expression(m[2]), cex=1.5)
text(xx[14], -2.3, expression(phi[2]), cex=1.5)
text(xx[15], -2.3, expression(mu^'-1'), cex=1.5)
legend(xx[9], -4, c("simulation value", "posterior mode"),
       pch=c(1,2), col=c("magenta", "blue"), bty="n", cex=1.5)
graphics.off()

postscript("pd_hf.ps", horizontal=FALSE, width=8, height=4, paper="special")
par(mfrow=c(1,2), pty="s", oma=c(0,0,1,1), mar=c(5,5,0,0))
max.pd <- max(c(pd[[1]], pd[[2]]))
plot(t, pd[[1]], ylim=c(0, max.pd), xlab="time", ylab="density",
     type="l", col="red")
lines(t, pd[[2]], col="blue")
legend(20, max.pd, legend=
       c(eval(substitute(expression(list(x[1] == a, x[2] == b)),
         list(a = x[1], b = x[3]))),
       eval(substitute(expression(list(x[1] == a, x[2] == b)),
         list(a = x[2], b = x[3])))),
       lty=1, col=c("red", "blue"), cex=.75, bty="n")
max.hf <- max(c(hf[[1]], hf[[2]]))
plot(t, hf[[1]], ylim=c(0, max.hf), xlab="time", ylab="hazard",
     type="l", col="red")
lines(t, hf[[2]], col="blue")
legend(20, 0.025, legend=
       c(eval(substitute(expression(list(x[1] == a, x[2] == b)),
         list(a = x[1], b = x[3]))),
       eval(substitute(expression(list(x[1] == a, x[2] == b)),
         list(a = x[2], b = x[4])))),
       lty=1, col=c("red", "blue"), cex=.75, bty="n")
graphics.off()

ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
