#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_sf_psi.h>
#include "../log_density.h"

void transform( double *, int *, double * );
void glm( double *, double *, int *, int *, double *, int * );

double post( double *theta, int *ip, double *dp )
{

   int i, mod, err;
   double pdf, sf1, sf2, tmp, wk, *ttheta, *p=theta, q[ip[1]-6], *x;
   gsl_error_handler_t *gsl_errh;

   gsl_errh = gsl_set_error_handler_off();

   if( ip[0] ) {
      ttheta = (double *) malloc( ip[1] * sizeof( double ) );
      ttheta = memcpy( ttheta, theta, ip[1] * sizeof( double ) );
      transform( ttheta, ip, dp );
      p = ttheta; 
   }
   else {

/* Boundaries for prior */

      for( i=0; i<ip[1]; i++ ) {
         if( p[i] <= dp[ip[7]+2*ip[8]+i] ||
             p[i] >= dp[ip[7]+2*ip[8]+ip[1]+i] ) {
            return( HUGE_VAL );
         }
      }
   }

   x = (double *) malloc( 2 * ip[8] * sizeof( double ) );
   for( i=0; i<2*ip[8]; i++ ) {
      x[i] = dp[ip[7]+i];
   }

/* Calculate likelihood */

   for( i=1; i<4; i++ ) {
      q[i] = p[i+1];
   }
   for( i=7; i<9; i++ ) {
      q[i] = p[i+6];
   }

   tmp = 0.0;
   for( i=0; i<ip[2]; i++ ) {
      mod = 1; glm( p, q, &ip[8], &ip[9+i], x, &mod );
      wk = 1.0 / sqrt( q[3] );
      pdf = log_invgauss_pdf( dp[i], wk, wk / q[2] );
      tmp += pdf + log( q[0] );
   }
   for( i=ip[2]; i<ip[3]; i++ ) {
      mod = 1; glm( p, q, &ip[8], &ip[9+i], x, &mod );
      mod = 2; glm( p, q, &ip[8], &ip[9+i], x, &mod );
      wk = 1.0 / q[5];
      pdf = log_gamma_pdf( dp[i], wk, wk / q[4] );
      tmp += pdf + log( 1.0 - q[0] );
   }
   for( i=ip[3]; i<ip[4]; i++ ) {
      mod = 3; glm( p, q, &ip[8], &ip[9+i], x, &mod );
      wk = 1.0 / q[7];
      pdf = log_gamma_pdf( dp[i], wk, wk / q[6] );
      tmp += pdf + log( q[1] );
   }
   for( i=ip[4]; i<ip[5]; i++ ) {
      pdf = log_exponential_pdf( dp[i], 1.0 / q[8] );
      tmp += pdf + log( 1.0 - q[1] );
   }
   for( i=ip[5]; i<ip[6]; i++ ) {
      mod = 1; glm( p, q, &ip[8], &ip[9+ip[4]-ip[5]+i], x, &mod );
      wk = 1.0 / sqrt( q[3] );
      invgauss_sf( dp[i], wk, wk / q[2], &sf1 );
      sf1 *= q[0];
      mod = 2; glm( p, q, &ip[8], &ip[9+ip[4]-ip[5]+i], x, &mod );
      wk = 1.0 / q[5];
      gamma_sf( dp[i], wk, wk / q[4], &sf2 );
      tmp += log( sf1 + (1.0 - q[0]) * sf2 );
   }
   for( i=ip[6]; i<ip[7]; i++ ) {
      mod = 3; glm( p, q, &ip[8], &ip[9+ip[4]-ip[5]+i], x, &mod );
      wk = 1.0 / q[7];
      gamma_sf( dp[i], wk, wk / q[6], &sf1 );
      sf1 *= q[1];
      sf2 = log_exponential_sf( dp[i], 1.0 / q[8] );
      sf2 = (1 - q[1]) * exp(sf2);
      tmp += log( sf1 + sf2 );
   }

   free( x );

/* Diffuse Prior */

   tmp += log_gauss_pdf( p[0], 0.0, 1e8 ) + log_gauss_pdf( p[1], 0.0, 1e8);
   tmp += log_gamma_pdf( q[2], 1.0, 1e-4 ) + log_gamma_pdf( q[3], 1.0, 1e-4 );
   tmp += log_gauss_pdf( p[5], 0.0, 1e8 ) + log_gauss_pdf( p[6], 0.0, 1e8) +
          log_gauss_pdf( p[7], 0.0, 1e8 ) + log_gauss_pdf( p[8], 0.0, 1e8) +
          log_gauss_pdf( p[9], 0.0, 1e8 ) + log_gauss_pdf( p[10], 0.0, 1e8) +
          log_gauss_pdf( p[11], 0.0, 1e8 ) + log_gauss_pdf( p[12], 0.0, 1e8);
   tmp += log_gamma_pdf( q[7], 1.0, 1e-4 ) + log_gamma_pdf( q[8], 1.0, 1e-4);

   gsl_set_error_handler( gsl_errh );

   if( ip[0] ) {
      free( ttheta );
   }

   err = gsl_finite( tmp );
   if( !err ) {
      return( HUGE_VAL );
   }

   return( -tmp );
}

void glm( double *p, double *q, int *nobs, int *i, double *x,
          int *c )
{
   switch( *c ) {
   case 1:
      q[0] = exp( p[0] + p[1] * x[*i] );
      q[0] /= (1.0 + q[0]);
      break;
   case 2:
      q[4] = exp( p[5] + p[6] * x[*i] + p[7] * x[*i+*nobs] );
      q[5] = exp( p[8] + p[9] * x[*i+*nobs] );
      break;
   case 3:
      q[6] = exp( p[10] + p[11] * x[*i] + p[12] * x[*i+*nobs] );
   }
}
