#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_qrng.h>

void nmsimplex( int *, int *, double *, double *, double *, double *,
                double *, int *, double *, int * );

void optimize( int *np, int *nstart, int *max_iter, double *sv, double *sp,
               double *size, double *ep, double *fv, int *ip, double *dp,
               int *err )
{

   int i, j;
   double *tep, tfv;
   gsl_qrng *q;

   tep = (double *) malloc( *np * sizeof(double) );
   q = gsl_qrng_alloc( gsl_qrng_sobol, *np );

   for( i=0; i<*nstart; i++ ) {
      fprintf( stderr, "Simplex = %d\n", i+1 );
      gsl_qrng_get( q, sv );
      for( j=0; j<*np; j++ ) {
         sv[j] = log( sv[j] ) - log( 1.0 - sv[j] );
      }
      nmsimplex( np, max_iter, sv, sp, size, tep, &tfv, ip, dp, err );
      if( *err ) {
         break;
      }
      if( i ) {
         if( tfv < *fv ) {
            *fv = tfv;
            for( j=0; j<*np; j++ ) {
               ep[j] = tep[j];
            }
         }
      }
      else {
         *fv = tfv;
         for( j=0; j<*np; j++ ) {
            ep[j] = tep[j];
         }
      }
   }

   free( tep ); gsl_qrng_free( q );
}

#include <stdlib.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_errno.h>

double post( double *, int *, double * );

void nmsimplex( int *np, int *max_iter, double *sv, double *sp, double *size,
                double *ep, double *fv, int *ip, double *dp, int *err )
{

   int i, iter=0, rval=GSL_CONTINUE, status=GSL_SUCCESS;
   double ssval, ll=0, eps=1e-8;

   const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;
   gsl_multimin_fminimizer *s = NULL;
   gsl_vector *ss, *x;
   gsl_multimin_function minex_func;
   gsl_error_handler_t *gsl_errh;
   struct fparams { double *theta; int *ip; double *dp; } fp;
   double f( const gsl_vector *v, void *p )
   {
      int i;
      struct fparams *fp = (struct fparams *)p;
      for( i=0; i<v->size; i++ ) {
         fp->theta[i] = gsl_vector_get( v, i );
      }
      return post( fp->theta, fp->ip, fp->dp );
   }

   fp.ip = ip; fp.dp = dp;
   fp.theta = (double *) malloc( *np * sizeof( double ) );

/* Initial vertex size vector */

   ss = gsl_vector_alloc( *np );

   if( ss == NULL ) {
      *err = 1; return;
   }

   for( i=0; i<*np; i++ ) {
      gsl_vector_set( ss, i, sv[i] );
   }

/* Starting point */

   x = gsl_vector_alloc( *np );

   if( x == NULL ) {
      gsl_vector_free( ss );
      *err = 2; return;
   }

   for( i=0; i<*np; i++ ) {
      gsl_vector_set( x, i, sp[i] );
   } 

/* Initialize method and iterate */

   minex_func.f = &f;
   minex_func.n = *np;
   minex_func.params = (void *) &fp;

   s = gsl_multimin_fminimizer_alloc( T, *np );
   gsl_multimin_fminimizer_set( s, &minex_func, x, ss );
   gsl_errh = gsl_set_error_handler_off();

   while( rval == GSL_CONTINUE && iter < *max_iter ) {
      iter++;
      status = gsl_multimin_fminimizer_iterate( s );

      if( ! (int) fmod( iter, 25 ) ) {
         if( abs( ll + s->fval ) <= eps ) {
            status = GSL_SUCCESS; rval = GSL_SUCCESS; break;
         }
         ll = -s->fval;
         fprintf( stderr, "\titer = %d, ll = %g\n", iter, ll );
      }

      if( status ) {
         break;
      }

      ssval = gsl_multimin_fminimizer_size( s );
      rval = gsl_multimin_test_size( ssval, *size );
   }

   if( status == GSL_SUCCESS ) {
      if( rval == GSL_SUCCESS ) {
         for( i=0; i<*np; i++ ) {
            ep[i] = gsl_vector_get( s->x, i );
         }
         *fv = s->fval;
      }
      else {
         *err = 3;
      }
   }
   else {
      *err = 4;
   }

   gsl_set_error_handler( gsl_errh );
   gsl_vector_free( x );
   gsl_vector_free( ss );
   gsl_multimin_fminimizer_free( s );
   free( fp.theta ); 
}
