optimize <- function (ntheta, ip, dp, nstart=10, nrestart=5, maxiter=500,
                      size=1e-2, eps=1e-2, sp=numeric(ntheta), f=transform,
                      outfile="", id=0)
{
   if (id) {
      system("make optimize.so; ln -s ../Makefile makefile;
              make -f makefile transform.so; rm makefile; make clean")
   } else {
      system("ln -s ../Makefile makefile; make -f makefile optimize.so;
              make -f makefile transform.so; make -f makefile clean")
   }
   dyn.load("optimize.so")

   ip <- c(1, ip);
   iv <- rep(1, ntheta); if.theta <- numeric(ntheta);
   ll <- 0; err <- 0;

   List <- .C("optimize", as.integer(ntheta), as.integer(nstart),
              as.integer(maxiter), as.double(iv), as.double(sp),
              as.double(size), if.theta = as.double(if.theta),
              ll = as.double(ll), as.integer(ip), as.double(dp),
              err = as.integer(err))

   err <- List$err
   if (err) {
      dyn.unload("optimize.so")
      if (id) {
         system("make clean.so")
      } else {
         system("make -f makefile clean.so; rm makefile")
      }
   }
   switch(err,
          stop("\nCannot allocate space for initial vertex vector.\n"),
          stop("\nCannot allocate space for starting point.\n"),
          stop("\nMaximum number of iterations for optimization exceeded.\n"),
          stop("\nError encountered in iterating optimizer.\n"))

   if.theta <- List$if.theta; ll <- List$ll; iv <- NULL; sp <- NULL;

   for( i in 1:nrestart ) {
      iv <- numeric(ntheta); sp <- if.theta; tmp.if.theta <- numeric(ntheta);
      tmp.ll <- 0; err <- 0;

      List <- .C("optimize", as.integer(ntheta), as.integer(nstart),   
                 as.integer(maxiter), as.double(iv), as.double(sp),
                 as.double(size), if.theta = as.double(tmp.if.theta),
                 ll = as.double(tmp.ll), as.integer(ip), as.double(dp),          
                 err = as.integer(err))

      err <- List$err
      if (err) {
         dyn.unload("optimize.so")
         if (id) {
            system("make clean.so")
         } else {
            system("make -f makefile clean.so; rm makefile")
         }
      }
      switch(err,
             stop("\nCannot allocate space for initial vertex vector.\n"),
             stop("\nCannot allocate space for starting point.\n"),
             stop("\nMaximum number of iterations for optimization exceeded.\n"),
             stop("\nError encountered in iterating optimizer.\n"))

      tmp.if.theta <- List$if.theta; tmp.ll <- List$ll; iv <- NULL; sp <- NULL;

      if( tmp.ll >= ll ) {
         print("Simplex restart does not improve initial solution.")
         break
      }
      else {
         est.eps <- 1 - tmp.ll/ll; if.theta <- tmp.if.theta; ll <- tmp.ll;
         if( est.eps <= eps ) {
            print("Simplex restart improvement does not exceed relative tolerance.")
            break
         }
      }
      print(paste("Simplex restart:  Iteration ", i, sep=""))
   }

   dyn.unload("optimize.so")
   if.theta <- f(if.theta, ip, dp)
   if (id) {
      system("make clean.so")
   } else {
      system("make -f makefile clean.so; rm makefile")
   }

   write("MAXIMUM LIKELIHOOD SUMMARY\n", file=outfile, append=TRUE)
   write("Solution:\n", file=outfile, append=TRUE)
   write(round(if.theta,4), file=outfile, ncolumns=ntheta, append=TRUE)
   write(paste("\nLog Likelihood:\t", round(-ll,4), sep=""), file=outfile,
         append=TRUE)

   list(theta=if.theta, ll=-ll)
}
