plots <- function( t=NULL, pd=NULL, sf=NULL, hf=NULL, theta=NULL,
                   theta.burnin=NULL, plot.mode=0, ulim=200,
                   xlab="Waiting Time", ylab=NULL, lwd=1, cex=1, cex.axis=1,
                   cex.lab=1, main=NULL,
                   plot=c("all", "pd", "cdf", "sf", "hf", "theta", "conv"))
{
   postscript(paste(plot, ".ps", sep=""), horizontal=FALSE)
   par(pty="s")
   switch(plot,
      pd = {
         plot(t, pd, xlab=xlab, ylab=ylab, main=main, type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
      },
      cdf = {
         plot(t, 1-sf, xlab=xlab, ylab=ylab, main=main, type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
      },
      sf = {
         plot(t, sf, xlab=xlab, ylab=ylab, main=main, type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
      },
      hf = {
         plot(t, hf, xlab=xlab, ylab=ylab, main=main, type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
      },
      all = {
         par(mfrow=c(2,2))
         plot(t, pd, xlab=xlab, ylab=ylab[1], main=main[1], type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
         plot(t, 1-sf, xlab=xlab, ylab=ylab[2], main=main[2], type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
         plot(t, sf, xlab=xlab, ylab=ylab[3], main=main[3], type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
         plot(t, hf, xlab=xlab, ylab=ylab[4], main=main[4], type="l",
              xlim=c(0, ulim), lwd=lwd, cex=cex, cex.axis=cex.axis,
              cex.lab=cex.lab)
      },
      theta = {
         for (i in 1:dim(theta)[2]) {
            hist(theta[,i], prob=TRUE, xlab=paste("Theta", i),
                 main=paste("Histogram of Theta", i))
            dT <- density(theta[,i],adjust=2)
            lines(dT, col="red")
            if (plot.mode || length(plot.mode) > 1) { abline(v=plot.mode[i],
                                                             col="blue") }
         }
      },
      conv = {
         dim.th.burn <- 0; dim.th <- dim(theta)[1];
         if (!is.null(theta.burnin)) { dim.th.burn <- dim(theta.burnin)[1]; }
         for (i in 1:dim(theta)[2]) {
            plot(1:(dim.th.burn+dim.th), c(theta.burnin[,i], theta[,i]),
                 xlab="Iteration", ylab=paste("Theta", i), type="l")
            if (plot.mode || length(plot.mode) > 1) { abline(h=plot.mode[i],
                                                             col="blue") }
            if (!is.null(theta.burnin)) { abline(v=dim(theta.burnin)[1],
                                                 col="red") }
         }
      }
   )
   graphics.off()
}
