#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

double calc_c( double, double * );
void sample_g( int, double *, double *, gsl_rng * );
double post( double *, int *, double * );
double g( double *, double * );

void rejection( double *sample, int *N, int *ntheta, int *seed, int *iter,
                double *fmax, double *par_g, int *ip, double *dp )
{

   int i, nsamp, fpf=1;
   double c, *theta, u;
   const gsl_rng_type *T;
   gsl_rng *r;

   T = gsl_rng_taus;
   r = gsl_rng_alloc( T );
   gsl_rng_set( r, *seed );

/* Set c for rejection sampling */

   c = calc_c( *fmax, par_g );

/* Rejection sampling */

   *iter = 0; nsamp = 0;
   theta = (double *) malloc( *ntheta * sizeof( double ) );
   while( nsamp < *N ) {
      if( (! (int) fmod( nsamp, 25 ) || nsamp == *N-1) && fpf ) {
         fpf = 0;
         fprintf( stderr, "Rejection sampling iterate = %d\n", nsamp );
      }
      if( (int) fmod( nsamp, 25 ) == 1 ) {
         fpf = 1;
      }

      (*iter)++;

/* 1.  Generate sample from g(.) */

      sample_g( *ntheta, theta, par_g, r );

/* 2.  Establish acceptance of sample */

      u = gsl_ran_exponential( r, 1.0 );
      if( u >= c + log( g( theta, par_g ) ) + post( theta, ip, dp ) ) {
         nsamp++;
         for( i=0; i<*ntheta; i++ ) {
            sample[i + (nsamp - 1) * *ntheta] = theta[i];
         }
      }
   } 

   gsl_rng_free( r );
   free( theta );
}
