#include <gsl/gsl_roots.h>
#include <gsl/gsl_errno.h>

void root( double *lb, double *ub, double *epsrel, double *epsabs,
           int *max_iter, gsl_root_fsolver *rf, gsl_function *F,
           double *s, int *err_rt )
{
   int iter, status;
   double x_lo=*lb, x_hi=*ub;
   gsl_error_handler_t *gsl_errh;

   gsl_errh = gsl_set_error_handler_off();

   iter = 0; *s = 0.0;
   gsl_root_fsolver_set( rf, F, x_lo, x_hi );

   do {
      iter++;
      status = gsl_root_fsolver_iterate( rf );
      *s = gsl_root_fsolver_root( rf );
      x_lo = gsl_root_fsolver_x_lower( rf );
      x_hi = gsl_root_fsolver_x_upper( rf );
      status = gsl_root_test_interval( x_lo, x_hi, *epsrel, *epsabs );
   }
   while( status == GSL_CONTINUE && iter < *max_iter );

   if( status == GSL_SUCCESS ) {
      *err_rt = 0;
   }
   else if( iter == *max_iter ) {
      *err_rt = 1;
   }
   else {
      *err_rt = 2;
   }
   gsl_set_error_handler( gsl_errh );

   return;
}
