#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <pthread.h>
#include <gsl/gsl_math.h>

void calc_sdi_ll( double *, double *, double *, double *, double *, double *,
                  double *, int *, double *, int * );
void saddlept( double *, double *, int *, double *, double *, int *,
               double *, double *, double *, int * );

void saddle( double *sp_lb, double *sp_ub, int *lt, double *t,
             double *theta, int *N, int *ntheta, int *max_iter,
             int *nthread, double *epsrel, double *epsabs,
             double *max_abserr, int *info, double *sp, double *nc,
             int *err_sp, int *file )
{
   int i, j, mm, MM, **errsp, iret[*nthread], NN[*lt];
   double **spd;
   pthread_t threads[*nthread];
   FILE *f;

   struct pthr { int N, ntheta, td, mm, MM, lt, max_iter, info, **err_sp;
                 double epsrel, epsabs, max_abserr, *theta, *t, *sp_lb,
                 *sp_ub, *nc, **sp; }
               thr[*nthread];
   void *dfeval( void *p )
   {
      int i, j, err_nc, *sig;
      double lb=0.0, *th, *w;
      struct pthr *pt = (struct pthr *) p;

      th = (double *) malloc( pt->ntheta * sizeof( double ) );
      w = (double *) malloc( pt->lt * sizeof( double ) );
      sig = (int *) calloc( pt->lt, sizeof( int ) );
      for( i=pt->mm; i<=pt->MM; i++ ) {
         for( j=0; j<pt->ntheta; j++ ) {
            th[j] = pt->theta[j+i*(pt->ntheta)];
         }

         saddlept( &pt->sp_lb[i], &pt->sp_ub[i], &pt->lt, pt->t, th,
                   &pt->max_iter, &pt->epsrel, &pt->epsabs, w, sig );
         for( j=0; j<pt->lt; j++ ) {
            if( !gsl_finite( w[j] ) || sig[j] ) { w[j] = 0.0; }
            sig[j] = 0;
         }

         calc_sdi_ll( &lb, &pt->sp_lb[i], &pt->sp_ub[i], &pt->epsrel,
                      &pt->epsabs, &pt->max_abserr, th, &pt->max_iter,
                      &pt->nc[i], &err_nc );
         if( pt->nc[i] <= 0.0 || !gsl_finite( pt->nc[i] ) || err_nc ) {
            pt->err_sp[i][j] = 1;
         }

         for( j=0; j<pt->lt; j++ ) {
            pt->sp[i][j] = w[j] / pt->nc[i];
         }
         if( pt->info && (! (int) fmod( i, 25 ) || i == pt->N-1) ) {
            fprintf( stderr, "Thread = %d, PDF Iterate = %d\n", pt->td, i );
         }
      }
      free( sig ); free( w ); free( th );
      pthread_exit( NULL );
   }

   errsp = (int **) malloc( *N * sizeof( int * ) );
   for( i=0; i<*N; i++ ) {
      errsp[i] = (int *) calloc( *lt, sizeof( int ) );
   }
   spd = (double **) malloc( *N * sizeof( double * ) );
   for( i=0; i<*N; i++ ) {
      spd[i] = (double *) calloc( *lt, sizeof( double ) );
   }
   for( i=0; i<*nthread; i++ ) {
      thr[i].ntheta = *ntheta; thr[i].theta = theta;
      thr[i].lt = *lt; thr[i].t = t;
      thr[i].sp_lb = sp_lb; thr[i].sp_ub = sp_ub; thr[i].nc = nc;
      thr[i].N = *N; thr[i].max_iter = *max_iter;
      thr[i].epsrel = *epsrel; thr[i].epsabs = *epsabs;
      thr[i].max_abserr = *max_abserr;
      thr[i].info = *info; thr[i].td=i+1;
      if( *nthread == 1 ) { mm = 0; MM = *N-1; }
      else {
         if( i == 0 ) { mm = 0; MM = *N / *nthread - 1; }
         else if( i == *nthread-1 ) { mm = MM+1; MM = *N-1; }
         else { mm = MM+1; MM = ((i+1) * (*N)) / *nthread - 1; }
      }
      thr[i].mm = mm; thr[i].MM = MM;
      thr[i].sp = spd; thr[i].err_sp = errsp;
      iret[i] = pthread_create( &threads[i], NULL, dfeval, (void *) &thr[i] );
   }
   for( i=0; i<*nthread; i++ ) {
      pthread_join( threads[i], NULL );
   }

   for( i=0; i<*lt; i++ ) {
      NN[i] = *N;
      for( j=0; j<*N; j++ ) {
         if( !errsp[j][i] ) { sp[i] += spd[j][i]; }
         else { NN[i]--; }
      }
      if( !NN[i] ) { *err_sp = 1; }
      if( *err_sp == 1 ) {
         for( j=0; j<*N; j++ ) { free( spd[j] ); free( errsp[j] ); }
         free( spd ); free( errsp );
         return;
      }
      sp[i] /= NN[i];
   }

   if( *file ) {
      f = fopen( "pd.out", "a" );
      for( i=0; i<*N; i++ ) {
         for( j=0; j<*lt-1; j++ ) {
            fprintf( f, "%g ", spd[i][j] );
         }
         fprintf( f, "%g\n", spd[i][*lt-1] );
         fflush( f );
      }
      fclose( f );
   }

   for( i=0; i<*N; i++ ) { free( spd[i] ); free( errsp[i] ); }
   free( spd ); free( errsp );
   return;
}
