saddle <- function (N, Theta, t, m=1e-6, M=1e6, nthread=1, maxiter=1000,
                    epsrel=1e-7, epsabs=0, max.abserr=1e-4, info=0,
                    file=FALSE)
{
   if (!is.matrix(Theta)) { Theta <- matrix(Theta, nrow=1) }

   system("ln -s ../Makefile makefile; make -f makefile calc_spb.so;
           make -f makefile saddle.so; make -f makefile clean")
   dyn.load("calc_spb.so")

   List <- .C("calc_spb", as.double(rep(m, N)), as.double(rep(M, N)),
              as.double(as.vector(t(Theta))), as.integer(N),
              as.integer(dim(Theta)[2]), as.integer(nthread),
              sp.lb=as.double(numeric(N)), sp.ub=as.double(numeric(N)))

   dyn.unload("calc_spb.so")

   sp.lb <- List$sp.lb; sp.ub <- List$sp.ub;

   dyn.load("saddle.so")

   List <- .C("saddle", as.double(sp.lb), as.double(sp.ub),
              as.integer(length(t)), as.double(t),
              as.double(as.vector(t(Theta))), as.integer(N),
              as.integer(dim(Theta)[2]), as.integer(maxiter),
              as.integer(nthread), as.double(epsrel),
              as.double(epsabs), as.double(max.abserr), as.integer(info),
              Sp=as.double(numeric(length(t))), nc=as.double(numeric(N)),
              err.sd=as.integer(0), as.integer(as.numeric(file)))

   dyn.unload("saddle.so")
   system("make -f makefile clean.so; rm makefile")

   err.sd <- List$err.sd
   if( err.sd ) {
      stop("\nError in calculating normalizing constant.\n")
   }

   list(pd=List$Sp, nc=List$nc, sp.lb=sp.lb, sp.ub=sp.ub)
}
