#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

double post( double *, int *, double * );
struct fparams { double *theta; int var; int *ip; double *dp; };
struct interval { double L; double R; };
void calc_I( int, double ( double, void * ), double, double, int,
             double, gsl_rng *, struct fparams *, struct interval * );
double x1( int, double ( double, void * ), double, double, double,
           gsl_rng *, struct fparams *, struct interval * );
double x1_or( int, double ( double, void * ), double, double, double,
              int, struct fparams *, struct interval * );

void slice( int *npar, int *N, int *m, int *b, int *method, int *seed,
            double *w, double *alpha, int *ip, double *dp, double *theta,
            int *file, double *sample )
{
   int i, j;
   double z, x0, u;
   struct fparams post_pars;
   double f( double x, void *p ) {
      struct fparams *fp = (struct fparams *) p;
      fp->theta[fp->var] = x;
      return( -post( fp->theta, fp->ip, fp->dp ) );
   }
   struct interval I;
   const gsl_rng_type *T;
   gsl_rng *r;
   FILE *of;

   /* Initialize */

   post_pars.ip = ip; post_pars.dp = dp;
   if( *file ) {
      of = fopen( "theta.out", "a" );
   }

   T = gsl_rng_taus;
   r = gsl_rng_alloc( T );
   gsl_rng_set( r, *seed );

   z = -post( theta, ip, dp ) - gsl_ran_exponential( r, 1.0 );

   /* Sample */

   for( i=0; i<*N; i++ ) {
      for( j=0; j<*npar; j++ ) {
         post_pars.theta = theta; post_pars.var = j; x0 = theta[j];
         calc_I( *method, f, x0, z, *m, *w, r, &post_pars, &I );
         u = gsl_rng_uniform_pos( r );
         if( u <= *alpha ) {
            theta[j] = x1( *method, f, x0, z, *w, r, &post_pars, &I );
         }
         else {
            theta[j] = x1_or( *method, f, x0, z, *w, *b, &post_pars, &I );
         }
         z = -post( theta, ip, dp ) - gsl_ran_exponential( r, 1.0 );
      }
      if( *file ) {
         for( j=0; j<*npar-1; j++ ) {
            fprintf( of, "%g ", theta[j] );
         }
         fprintf( of, "%g\n", theta[*npar-1] );
         fflush( of );
      }
      else {
         for( j=0; j<*npar; j++ ) {
            sample[j + i * *npar] = theta[j];
         }
      }
      if( ! (int) fmod( i, 25 ) || i == *N-1 ) {
         fprintf( stderr, "Slice iterate = %d\n", i );
      }
   }
   if( *file ) {
      fclose( of );
   }
   gsl_rng_free( r );
}

void step_out( double f( double, void * ), double, double, int, double,
               gsl_rng *, struct fparams *, struct interval * );
void doubling( double f( double, void * ), double, double, int, double,
               gsl_rng *, struct fparams *, struct interval * );

void calc_I( int meth, double f( double, void * ), double x, double z,
             int m, double w, gsl_rng *r, struct fparams *post_pars,
             struct interval *I )
{
   if( meth == 1 ) {
      step_out( f, x, z, m, w, r, post_pars, I );
   }
   else if( meth == 2 ) {
      doubling( f, x, z, m, w, r, post_pars, I );
   }
}

void step_out( double f( double, void * ), double x, double z, int m,
               double w, gsl_rng *r, struct fparams *post_pars,
               struct interval *I )
{
   int J, K;
   double u;

   u = gsl_rng_uniform_pos( r );   
   I->L = x - w * u;
   I->R = I->L + w;

   u = gsl_rng_uniform_pos( r );
   J = (int) (m * u);
   K = (m - 1) - J;

   while( J > 0 && z < f( I->L, post_pars ) ) {
      I->L -= w; J--;
   }

   while( K > 0 && z < f( I->R, post_pars ) ) {
      I->R += w; K--;
   }
}

void doubling( double f( double, void * ), double x, double z, int m,
               double w, gsl_rng *r, struct fparams *post_pars,
               struct interval *I )
{
   int K;
   double u, fL, fR;

   u = gsl_rng_uniform_pos( r );
   I->L = x - w * u;
   I->R = I->L + w;
   K = m;

   fL = f( I->L, post_pars ); fR = f( I->R, post_pars );
   while( K > 0 && ( z < fL || z < fR ) ){
      u = gsl_rng_uniform_pos( r );
      if( u < 0.5 ) {
         I->L -= (I->R - I->L); fL = f( I->L, post_pars );
      }
      else {
         I->R += (I->R - I->L); fR = f( I->R, post_pars );
      } 
      K--;
   }
}

int test_A( int, double f( double, void * ), double, double, double,
            double, struct fparams *, struct interval * );

double x1( int meth, double f( double, void * ), double x0, double z,
           double w, gsl_rng *r, struct fparams *post_pars, struct interval *I )
{
   double L, R, u, x1;

   L = I->L; R = I->R;
   while( 1 ) {
      u = gsl_rng_uniform_pos( r );
      x1 = L + u * (R - L);
      if( z < f( x1, post_pars ) &&
          test_A( meth, f, x0, x1, z, w, post_pars, I ) ) {
         return( x1 );
      }
      if( x1 < x0 ) {
         L = x1;
      }
      else {
         R = x1;
      }
   }
}

double x1_or( int meth, double f( double, void * ), double x0, double z,
              double w, int b, struct fparams *post_pars, struct interval *I )
{
   double L, R, v, M, x1;

   L = I->L; R = I->R; v = w;
   if( R - L < 1.1 * w ) {
      while( 1 ) {
         M = (L + R) * 0.5;
         if( !b || z < f( M, post_pars ) ) {
            break;
         }
         if( x0 > M ) {
            L = M;
         }
         else {
            R = M;
         }
         b--; v /= 2;
      }
   }
   while( b > 0 ) {
      b--; v /= 2;
      if( z >= f( L + v, post_pars ) ) {
         L = L + v;
      }
      if( z >= f( R - v, post_pars ) ) {
         R = R - v;
      }
   }

   x1 = L + R - x0;
   if( x1 < L || x1 > R || z >= f( x1, post_pars ) ||
       !test_A( meth, f, x0, x1, z, w, post_pars, I ) ) {
      x1 = x0;
   }
   return( x1 );
}

int test_A( int meth, double f( double, void * ), double x0, double x1,
            double z, double w, struct fparams *post_pars, struct interval *I )
{

   int D;
   double L, R, M;

   if( meth == 1 ) {
      return( 1 );
   }
   else {
      L = I->L; R = I->R; D = 0;

      while( R - L > 1.1 * w ) {
         M = 0.5 * (L + R);
         if( ( x0 < M && x1 >= M ) || ( x0 >= M && x1 < M ) ) {
            D = 1;
         }
         if( x1 < M ) {
            R = M;
         }
         else {
            L = M;
         }
         if( D && z >= f( L, post_pars ) && z >= f( R, post_pars ) ) {
            return( 0 );
         }
      }
      return( 1 );
   }
}
