slice <- function ( N, ntheta, ip, dp, start, m=100, b=5, nburn=1000, w=0.01,
                    alpha=0.05, seed=round((2^16-1)*runif(1),0), meth=1,
                    outfile="", id=0, file=FALSE )
{
   if (id) {
      system("make slice.so; make clean")
   } else {
      system("ln -s ../Makefile makefile; make -f makefile slice.so;
              make -f makefile clean")
   }
   dyn.load("slice.so")

   ip <- c(0, ip);
   if (!file ) {
      sample <- numeric((N+nburn) * ntheta)
   } else {
      sample <- numeric(1)
   }
   if (!is.matrix(start)) { start <- matrix(start, nrow=1) }

   Slice.Sample <- list()
   for (i in 1:dim(start)[1]) {
      List <- .C("slice", as.integer(ntheta), as.integer(N+nburn), as.integer(m),
                 as.integer(b), as.integer(meth), as.integer(seed),
                 as.double(w), as.double(alpha), as.integer(ip), as.double(dp),
                 as.double(start[i,]), as.integer(as.numeric(file)),
                 Theta = as.double(sample))
      if (!file) {
         Slice.Sample[[i]] <- t(matrix(List$Theta, nrow=ntheta))
      } else {
         Slice.Sample[[i]] <- NULL
      }
   }

   dyn.unload("slice.so")
   if (id) {
      system("make clean.so")
   } else {
      system("make -f makefile clean.so; rm makefile")
   }

   write("\n\nSLICE SAMPLING SUMMARY\n", file=outfile, append=TRUE)
   write(paste("Number of replicates:\t", dim(start)[1], sep=""),
         file=outfile, append=TRUE)
   write(paste("Number of burn-in samples:\t", nburn, sep=""),
         file=outfile, append=TRUE)
   write(paste("Number of posterior samples:\t", N, sep=""),
         file=outfile, append=TRUE)

   Slice.Sample
}
