sumstats <- function (N, Theta, t, fid, M=100, A=20, maxiter.fi=10,
                      eps=1e-8, maxiter.rf=1000, epsrel=1e-5, epsabs=1e-6,
                      max.abserr=1e-4, task=0, cdf=numeric(length(t)),
                      p=c(0.25,0.5,0.75,0.9,0.95,0.99), nthread=1,
                      outfile="ss.out", vb=TRUE, compile=TRUE, clean=TRUE,
                      file=FALSE)
{
   if (vb && task != 2) {
      write("\n\nPREDICTIVE DISTRIBUTION SUMMARY STATISTICS\n", file=outfile,
            append=TRUE)
   }

   if (!is.matrix(Theta)) { Theta <- matrix(Theta, nrow=1) }

   if (compile) {
      system("ln -s ../Makefile makefile; make -f makefile sumstats_fi.so;
              make -f makefile clean")
   }
   dyn.load("sumstats.so")

   List <- .C("sumstats", as.integer(M), as.double(A), as.integer(length(t)),
              as.double(t), as.integer(dim(Theta)[2]),
              as.double(as.vector(t(Theta))), as.integer(N),
              as.integer(maxiter.fi), as.integer(maxiter.rf), as.integer(nthread),
              as.double(epsrel), as.double(epsabs), as.double(max.abserr),
              as.double(eps), as.double(fid), as.double(p),
              as.integer(length(p)), as.integer(task),
              as.integer(as.numeric(vb)), vdk=as.double(numeric(N)),
              vddk=as.double(numeric(N)), mu=as.double(0),
              var=as.double(0), cdf=as.double(cdf), t.ub=as.double(0),
              hf=as.double(numeric(length(t))),
              q=as.double(numeric(length(p))), err.ss=as.integer(0),
              as.integer(as.numeric(file)))

   dyn.unload("sumstats.so")
   if (clean) {
      system("make -f makefile clean.so; rm makefile")
   }

   err.ss <- List$err.ss
   switch(err.ss,
      stop("\nError in calculating CDF.\n"),
      stop("\nMaximum iteration limit reached in root finder.\n"),
      stop("\nError in root finder.\n"))

   if (vb && (!task || task == 1)) {
      write(paste("Mean:\t", round(List$mu, 2), sep=""), file=outfile,
            append=TRUE)
      write(paste("Standard Deviation:\t", round(sqrt(List$var), 2), "\n",
                  sep=""), file=outfile, append=TRUE)
   }

   if (vb && (!task || task == 3)) {
      for( i in 1:length(List$q) ) {
         write(paste(round(100*p[i], 2), "%-tile:\t", round(List$q[i], 2),
               sep=""), file=outfile, append=TRUE)
      }
   }

   if (List$t.ub <= 0) { List$t.ub <- NULL; }

   list(sf=1.0-List$cdf, xlim.up=List$t.ub, hf=List$hf, q=List$q,
        vdk=List$vdk, vddk=List$vddk, mu=List$mu, var=List$var)
}
