#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <pthread.h>
#include <gsl/gsl_roots.h>
#include <gsl/gsl_errno.h>

void moments( int *, int *, double *, double *, double *, double *, double * );
void calc_sdi_lu( double *, double *, double *, double *, double *, double *,
                  double *, double *, int *, double *, int * );
void root( double *, double *, double *, double *, int *, gsl_root_fsolver *,
           gsl_function *, double *, int * );

void sumstats( double *sp_lb, double *sp_ub, int *lt, double *t, int *ntheta,
               double *theta, double *nc, int *N, int *max_iter_sp,
               int *max_iter_rf, int *nthread, double *epsrel, double *epsabs,
               double *max_abserr, double *pd, double *p, int *np, int *task,
               int *vb, double *vdk, double *vddk, double *mu, double *var,
               double *cdf, double *t_ub, double *hf, double *q, int *err_ss,
               int *file )
{
   int i, j, mm, MM, nit, **errsd, *errss, iret[*nthread], NN[*lt];
   double **ccdf;
   pthread_t threads[*nthread];
   FILE *of;

   struct fparams { double lb, ub, fmax, epsrel, epsabs, max_abserr,
                    prob, *sp_lb, *sp_ub, *theta, *th, *nc; int ii, N,
                    ntheta, max_iter, err_sdi; } cdf_eval;
   struct pthr { int td, mm, MM, lt, vb, **err_ss; double *t, **cdf;
                 struct fparams *cdf_eval; } thr[*nthread];
   struct qthr { int td, mm, MM, lt, vb, max_iter, *err_ss; double *t,
                 *cdf, *p, *q; struct fparams *cdf_eval; } tthr[*nthread];
   double f0( double x, void *p )
   {
      double cdf;
      struct fparams *fp = (struct fparams *) p;

      cdf = 0.0;
      calc_sdi_lu( &fp->lb, &x, &fp->sp_lb[fp->ii], &fp->sp_ub[fp->ii],
                   &fp->epsrel, &fp->epsabs, &fp->max_abserr, fp->th,
                   &fp->max_iter, &cdf, &fp->err_sdi );
      return( cdf );
   }
   double f( double x, void *p )
   {
      int i, j;
      double cdf;
      struct fparams *fp = (struct fparams *) p;

      cdf = 0.0;
      for( i=0; i<fp->N; i++ ) {
         for( j=0; j<fp->ntheta; j++ ) {
            fp->th[j] = fp->theta[j+i*(fp->ntheta)];
         }
         fp->ii = i;
         cdf += (f0( x, fp ) / fp->nc[i]);
      }
      return( cdf / fp->N );
   }
   double g( double x, void *p )
   {
      struct fparams *fp = (struct fparams *) p;
      if( x == fp->lb ) { return( -fp->prob ); }
      else if( x == fp->ub ) { return( fp->fmax - fp->prob ); }
      else { return( f( x, fp ) - fp->prob ); }
   }
   void *cdfeval( void *p )
   {
      int i,j;
      struct pthr *pt = (struct pthr *) p;
      struct fparams fpc;

      fpc.lb = pt->cdf_eval->lb;
      fpc.epsrel = pt->cdf_eval->epsrel; fpc.epsabs = pt->cdf_eval->epsabs;
      fpc.max_abserr = pt->cdf_eval->max_abserr;
      fpc.sp_lb = pt->cdf_eval->sp_lb; fpc.sp_ub = pt->cdf_eval->sp_ub;
      fpc.ntheta = pt->cdf_eval->ntheta; fpc.theta = pt->cdf_eval->theta;
      fpc.th = (double *) malloc( fpc.ntheta * sizeof( double ) );
      fpc.nc = pt->cdf_eval->nc; fpc.N = pt->cdf_eval->N;
      fpc.max_iter = pt->cdf_eval->max_iter;
      fpc.err_sdi = pt->cdf_eval->err_sdi;

      for( i=pt->mm; i<=pt->MM; i++ ) {
         for( j=0; j<fpc.ntheta; j++ ) {
            fpc.th[j] = fpc.theta[j+i*(fpc.ntheta)];
         }
         fpc.ii = i;
         for( j=0; j<pt->lt; j++ ) {
            pt->cdf[i][j] = f0( pt->t[j], &fpc ) / fpc.nc[i];
            if( fpc.err_sdi ) { pt->err_ss[i][j] = 1; }
         }
         if( pt->vb && (! (int) fmod( i, 25 ) || i == fpc.N-1 )) {
            fprintf( stderr, "Thread = %d, CDF Iterate = %d\n", pt->td, i );
         }
      }
      free( fpc.th );
      pthread_exit( NULL );
   }
   void *qcdf( void *p )
   {
      int i, j, err_rt;
      double x_lo, x_hi, f_lo, f_hi;
      const gsl_root_fsolver_type *T;
      gsl_root_fsolver *rf;
      gsl_function F;
      struct qthr *pt = (struct qthr *) p;
      struct fparams fpc;

      fpc.epsrel = pt->cdf_eval->epsrel; fpc.epsabs = pt->cdf_eval->epsabs;
      fpc.max_abserr = pt->cdf_eval->max_abserr;
      fpc.sp_lb = pt->cdf_eval->sp_lb; fpc.sp_ub = pt->cdf_eval->sp_ub;
      fpc.ntheta = pt->cdf_eval->ntheta; fpc.theta = pt->cdf_eval->theta;
      fpc.th = (double *) malloc( fpc.ntheta * sizeof( double ) );
      fpc.nc = pt->cdf_eval->nc; fpc.N = pt->cdf_eval->N;
      fpc.max_iter = pt->cdf_eval->max_iter;
      fpc.err_sdi = pt->cdf_eval->err_sdi;

      T = gsl_root_fsolver_brent;
      rf = gsl_root_fsolver_alloc( T );
      F.function = &g; F.params = &fpc;

      for( i=pt->mm; i<=pt->MM; i++ ) {
         j = 0;
         while( j < pt->lt && pt->cdf[j] <= pt->p[i] ) {
            j++;
         }
         x_lo = pt->t[j-1]; f_lo = pt->cdf[j-1];
         if( j == pt->lt ) {
            fpc.lb = pt->cdf_eval->lb;
            x_hi = 2.0 * x_lo;
            f_hi = f( x_hi, &fpc );
            while( f_hi <= pt->p[i] ) {
               x_lo = x_hi; f_lo = f_hi; x_hi *= 2.0;
               f_hi = f( x_hi, &fpc );
            }
         }
         else {
            x_hi = pt->t[j]; f_hi = pt->cdf[j];
         }
         fpc.lb = x_lo; fpc.ub = x_hi; fpc.fmax = f_hi - f_lo;
         fpc.prob = pt->p[i] - f_lo;

         root( &x_lo, &x_hi, &fpc.epsrel, &fpc.epsabs, &pt->max_iter,
               rf, &F, &pt->q[i], &err_rt );

         if( !err_rt ) {
            pt->err_ss[i] = 0;
         }
         else if( err_rt == 1 ) {
            pt->err_ss[i] = 2;
         }
         else {
            pt->err_ss[i] = 3;
         }
         if( pt->vb ) {
            fprintf( stderr, "Thread = %d, i = %d, q = %g\n", pt->td,
                     i, pt->q[i] );
         }
      }
      gsl_root_fsolver_free( rf );
      free( fpc.th );
      pthread_exit( NULL );
   }

   if( !*task || *task == 1 ) {
      moments( N, ntheta, theta, vdk, vddk, mu, var );
      if( *task == 1 ) {
         return;
      }
   }

   cdf_eval.lb = 0.0;
   cdf_eval.epsrel = *epsrel; cdf_eval.epsabs = *epsabs;
   cdf_eval.max_abserr = *max_abserr;
   cdf_eval.sp_lb = sp_lb; cdf_eval.sp_ub = sp_ub;
   cdf_eval.ntheta = *ntheta; cdf_eval.theta = theta;
   cdf_eval.nc = nc; cdf_eval.N = *N;
   cdf_eval.max_iter = *max_iter_sp; cdf_eval.err_sdi = 0;

   if( !*task || *task == 2 ) {
      errsd = (int **) malloc( *N * sizeof( int * ) );
      for( i=0; i<*N; i++ ) {
         errsd[i] = (int *) calloc( *lt, sizeof( int ) );
      }
      ccdf = (double **) malloc( *N * sizeof( double * ) );
      for( i=0; i<*N; i++ ) {
         ccdf[i] = (double *) calloc( *lt, sizeof( double ) );
      }
      for( i=0; i<*nthread; i++ ) {
         thr[i].lt = *lt; thr[i].vb = *vb; thr[i].t = t;
         thr[i].cdf_eval = &cdf_eval; thr[i].td=i+1;
         if( *nthread == 1 ) { mm = 0; MM = *N-1; }
         else {
            if( i == 0 ) { mm = 0; MM = *N / *nthread - 1; }
            else if( i == *nthread-1 ) { mm = MM+1; MM = *N-1; }
            else { mm = MM+1; MM = ((i+1) * (*N)) / *nthread - 1; }
         }
         thr[i].mm = mm; thr[i].MM = MM;
         thr[i].cdf = ccdf; thr[i].err_ss = errsd;
         iret[i] = pthread_create( &threads[i], NULL, cdfeval, (void *) &thr[i] );
      }
      for( i=0; i<*nthread; i++ ) {
         pthread_join( threads[i], NULL );
      }

      for( i=0; i<*lt; i++ ) {
         NN[i] = *N;
         for( j=0; j<*N; j++ ) {
            if( !errsd[j][i] ) { cdf[i] += ccdf[j][i]; }
            else { NN[i]--; }
         }
         if( !NN[i] ) { *err_ss = 1; }
         if( *err_ss == 1 ) {
            for( j=0; j<*N; j++ ) { free( ccdf[j] ); free( errsd[j] ); }
            free( ccdf ); free( errsd );
            return;
         }

         cdf[i] /= NN[i];
         if( cdf[i] >= 1.0 ) {
            *t_ub = t[i]; cdf[i] = 1.0; hf[i] = 0.0; break;
         }
         else {
            hf[i] = (pd[i] <= 0.0) ? 0.0 : (exp( log( pd[i] ) -
                     log( 1.0 - cdf[i] ) ) );
         }
      }
      if( i < (*lt-1) ) {
         for( j=i+1; j<*lt; j++ ) {
            cdf[j] = 1.0; hf[j] = 0.0;
         }
      }

      if( *file ) {
         of = fopen( "cdf.out", "a" );
         for( i=0; i<*N; i++ ) {
            for( j=0; j<*lt-1; j++ ) {
               fprintf( of, "%g ", ccdf[i][j] );
            }
            fprintf( of, "%g\n", ccdf[i][*lt-1] );
            fflush( of );
         }
         fclose( of );
      }

      for( i=0; i<*N; i++ ) { free( ccdf[i] ); free( errsd[i] ); }
      free( ccdf ); free( errsd );

      if( *task == 2 ) {
         return;
      }
   }

   if( !*task || *task == 3 ) {
      errss = (int *) calloc( *np, sizeof( int ) );
      nit = (*nthread > *np) ? *np : *nthread;
      for( i=0; i<nit; i++ ) {
         tthr[i].lt = *lt; tthr[i].t = t;
         tthr[i].vb = *vb; tthr[i].max_iter = *max_iter_rf;
         tthr[i].cdf_eval = &cdf_eval; tthr[i].td=i+1;
         if( *nthread == 1 ) { mm = 0; MM = *np-1; }
         else {
            if( i == 0 ) { mm = 0; MM = *np / nit - 1; }
            else if( i == nit-1 ) { mm = MM+1; MM = *np-1; }
            else { mm = MM+1; MM = ((i+1) * (*np)) / nit - 1; }
         }
         tthr[i].mm = mm; tthr[i].MM = MM;
         tthr[i].cdf = cdf; tthr[i].err_ss = errss;
         tthr[i].p = p; tthr[i].q = q;
         iret[i] = pthread_create( &threads[i], NULL, qcdf, (void *) &tthr[i] );
      }
      for( i=0; i<nit; i++ ) {
         pthread_join( threads[i], NULL );
      }

      for( i=0; i<*np; i++ ) {
         if( errss[i] ) {
            *err_ss = errss[i]; free( errss ); return;
         }
      }
      free( errss );
   }

   return;
}
