#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../rejection.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../plots.r")

#
# Set-up
#

outsum <- "summary_rs"
ntheta <- 3; N <- 2000;
m <- 1e-6; M <- 1e6;
xlim.up <- 200
t <- seq(m, xlim.up, 2)

#
# Read the data
#

dat.df <- read.table("TRANS1.DAT", header=FALSE)

t12 <- dat.df[,4] - dat.df[,2]
tmp <- dat.df[,6] - dat.df[,4]
t23 <- tmp[dat.df[,7] == 3]; c2 <- tmp[dat.df[,7] == 2];

ip <- numeric(); ip[1] <- ntheta;
ip[2] <- length(t12); ip[3] <- ip[2]+length(t23);
ip[4] <- ip[3]+length(c2); 

dp <- c(t12, t23, c2)

lb <- c(1/35, 1.5, 1/5.5)
ub <- c(1/22, 6, 1/1.5)
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

#
# Find MAP
#

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Rejection sampling from posterior
#

par.g <- prior.p
par.g[2*ntheta+1] <- 1
for( i in 1:ntheta ) {
   par.g[2*ntheta+1] <- par.g[2*ntheta+1] /
                        (par.g[ntheta+i] - par.g[i])
}
Theta.rs <- rejection(N, ntheta, MAP$ll, par.g, ip, dp, outfile=outsum)

write.table(round(Theta.rs, 5), file="Theta.rs", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

Theta <- Theta.rs

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Summary statistics
#

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd,
                  nthread=3, outfile=outsum)
sf <- stats$sf; hf <- stats$hf; q <- stats$q;
xlim.up <- min(c(xlim.up, stats$xlim.up, q[length(q)]))

save.image()

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, sf=sf, hf=hf, ylab=c("density", "cdf", "survivor",
      "hazard"), ulim=xlim.up, plot="all")
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="cdf", ulim=xlim.up,
      lwd=2, plot="cdf")
plots(t, sf=sf, xlab="t", ylab="survivor", ulim=xlim.up,
      lwd=2, plot="sf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
